%
%   PicoMat RTI - A real-time interface between the PicoScope, Matlab, and a Webcam
%               - Designed to allow undergraduate students to point a webcam using thoughts alone!
%
%   Copyright (c) 2012 by Professor Charles M. Higgins, Dept. of Neuroscience, U. Arizona
%   All rights reserved.
%
%   Redistribution and use in source and binary forms, with or without modification, 
%   are permitted provided that the following conditions are met:
%
%   Redistributions of source code must retain the above copyright notice, this list of 
%   conditions and the following disclaimer. Redistributions in binary form must reproduce 
%   the above copyright notice, this list of conditions and the following disclaimer in the 
%   documentation and/or other materials provided with the distribution.  Neither the name 
%   of the University of Arizona nor the names of its contributors may be used to endorse or 
%   promote products derived from this software without specific prior written permission.
%
%   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
%   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
%   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
%   THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
%   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
%   OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
%   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
%   TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
%   EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%
%   If you do use this software, I ask that you leave my name 
%   (Professor Charles M. Higgins) as author.
%
% History:
% -Original 1-channel code PicoMatlab.m from The Mathworks for PicoScope 2000
% -Extensively modified for PicoScope 3245 for NROS 415 by Charles M Higgins, August 2012
% -Fixed, extended to 4 channels, autoscale, process ephys data, untold features, CMH November 2012
% -Added audio mode, many fixes and enhancements, CMH 11/22/12
% v1.5: Added multi-Picoscope mode (huge changes), CMH 11/27/12
%
% PicoMat_RTI(argument,argument2)
%
% -Send no arguments for an interactive dialog (most people will want to do this)
% -Send 0 or no argument for 'PicoScope mode'
%     Any OTHER mode disables use of PicoScope hardware, drivers!
% -Send an argument of (0, num_pico) to open only num_pico out of the attached picoscopes
% -Send an argument of 1 for 'Simulation random mode': random data (0-1) times 1V sinusoid at 0.2 Hz,
%	  4 channels out of phase by pi/2
% -Send an argument of 2 for 'Simulation frequency sweep mode': create data sweeping from 0.5 to 60 Hz
% -Send an argument of 3 for 'Recorded data mode': you will get a popup dialog
% -Send only a filename argument for 'Recorded data mode': load data from file and play as if in real time
% -Send an argument of 4 for 'Audio data mode': load audio, audio-style spectrum analysis
%
%
% Technical note from CMH, 11/20/12
% Significant design concern: the time between PicoScope buffers is not sampled.  What artifacts
%   will this result in when multiple buffers are combined long-term?  Probably at least a 
%   frequency component at the buffer frequency; hopefully, too high to worry about for EEG.
%

function PicoMat_RTI(argument,argument2)

% Try to reduce persistence from previous runs, but loses arguments
if exist('argument') 
	save('.PMRTItemp1.mat','argument');
end;
if exist('argument2') 
	save('.PMRTItemp2.mat','argument2');
end;
clear all;
clear global;
if exist('.PMRTItemp1.mat','file')
	load('.PMRTItemp1.mat');
	delete('.PMRTItemp1.mat');
end;
if exist('.PMRTItemp2.mat','file')
	load('.PMRTItemp2.mat');
	delete('.PMRTItemp2.mat');
end;

try
	% Clumsily pass on arguments to wrapper function (better to use varargin)
	if exist('argument') 
		if exist('argument2') 
			PicoMat_RTI_main(argument,argument2)
		else
			PicoMat_RTI_main(argument);
		end;
	else
			PicoMat_RTI_main();
	end;
catch err
    uiwait(msgbox({'PicoMAT RTI has had an new and improved error of some kind.','Please report how you produced this error to Professor Higgins.'},'modal'));
	err
%	throw(err);
	EXIT_prog(0,0);
	return;
end;


function PicoMat_RTI_main(argument,argument2)

%if exist('argument')
%	argument
%else
%	fprintf('argument doesn''t exist.\n');
%end;
%if exist('argument2')
%	argument2
%else
%	fprintf('argument2 doesn''t exist.\n');
%end;

% Latest code version
version = '1.8';
version_date = '11/28/12';

global use_picoscope;
global mode; % 0=PicoScope, 1=Simulation-random, 2=Simulation-sweep, 3=Recorded-data
global input; % used only in mode 3
global program_name;
global modename;
program_name = ['PicoMat RTI v' version];
if ~isempty(strfind(mexext,'mac'))
	% <strong> doesn't work on mac, but fine on PC.  Use it when we can.
	s1 = '';
	s2 = '';
else
	s1 = '<strong>';
	s2 = '</strong>';
end;
long_program_name1 = [s1 'PicoMat RTI' s2 ': ' s1 'Pico' s2 'scope ' s1 'Mat' s2 'lab ' s1 'R' s2 'eal-' s1 'T' s2 'ime ' s1 'I' s2 'nterface version ' version ','];
long_program_name2 = ['       by ' s1 'Charles M. Higgins' s2 '. Last updated ' version_date '.'];
clc;
fprintf(2,'%s\n',long_program_name1);
fprintf(2,'%s\n',long_program_name2);

modenames = {'PicoScope mode','Simulation random mode','Simulation sweep mode','Recorded data mode','Audio data mode'};
up =        [1,                0,                       0,                     0,                    0];

% Make sure program is not already running (causes callback problems)
f1 = findobj('type','figure','name','Raw data window');
f2 = findobj('type','figure','name','Raw data window 1');
if ~isempty(f1) || ~isempty(f2)
	if ~isempty(f1)
		figure(f1);
	else
		figure(f2);
	end;
    msgbox({'PicoMAT RTI is already running; press EXIT on any open PicoMat RTI first.','If PicoMat RTI is not running, try ''close all force''.'});
    return;
end;

% Create a dialog with mode choices for the user
if ~exist('argument')  
	[selection,ok] = listdlg('ListString',modenames,'SelectionMode','Single','ListSize',[230 80],...
								'Name',program_name,'PromptString','Which mode would you like to use?');
	if ok
		argument = selection-1;
		if argument == 0
			clear argument;
		end;
	else
		% User pressed cancel.  Exit.
		return;
	end;
end;
	
max_pico = 4;
if ~exist('argument')  
	mode = 0;
    % Check if computer supports this mode
    if ~isempty(strfind(mexext,'mac'))
        % Macintosh computer, don't bother trying to load DLL
        uiwait(msgbox({'This computer is a Macintosh, for which PicoScope mode does not work.','Switching to Simulation random mode.','Despite this minor setback, be assured you are using a superior computing machine.'},'modal'));
        mode = 1;
    elseif ~isempty(strfind(mexext,'64'))
        uiwait(msgbox({'You are running the 64-bit version of Matlab, for which PicoScope mode does not work.','Switching to Simulation random mode.','Please run the 32-bit version of Matlab to use PicoScope mode.'},'modal'));
        mode = 1;
    end;
	modename = cell2mat(modenames(mode+1));
	use_picoscope = up(mode+1);
else
	% Argument exists
	if isnumeric(argument)
		if argument == 0
			mode = 0;
			% Check for second argument
			if exist('argument2') && isnumeric(argument2)
				max_pico = argument2;
			end;
		elseif argument == 1
			mode = 1;
		elseif argument == 2
			mode = 2;
		elseif argument == 3
			mode = 3;
		elseif argument == 4
			mode = 4;
		else
			s = sprintf('Sorry, mode %d doesn''t make any sense to me.',argument);
        	uiwait(msgbox({s,'Switching to Simulation random mode.'},'modal'));
			mode = 1; % Default mode
		end;
		modename = cell2mat(modenames(mode+1));
		use_picoscope = up(mode+1);
	end;
	if myequal(mode,3) || ~isnumeric(argument)
		% Argument exists, but is not numeric; try to load filename
		mode = 3;
		modename = cell2mat(modenames(mode+1));
		use_picoscope = up(mode+1);
		if isnumeric(argument)
			% 3 provided, but no filename. Put a filename in 'input' from the user or exit
			[name path success] = uigetfile('*.mat','Select a file to load');
			if (success)
				if strcmp(pwd,path(1:end-1))
					% Don't need path if in same dir
					filename = [name];
				else
					filename = [path name];
				end;
			else
				return;
			end;
		else
			filename = argument;
		end;
		try
			input = load(filename,'-mat');
		catch err
			s = sprintf('Sorry, could not load file ''%s''.\n',filename);
	    	msgbox(s);
			return;
		end;
	end;
end;

%% Check for resample
%try
%	v = pepresample(out_data,p,q);
%catch err
%   	uiwait(msgbox({'Hmm, the resample function doesn''t exist.','Looks like you''re missing the signal processing toolbox.','You must install it to use Recorded data mode.'},'modal'));
%	EXIT_prog(0,0);
%	return;
%end;


fprintf('%s',modename);
if mode == 0 || mode == 4
	fprintf(': ');
else
	fprintf('\n');
end;


global truncate_at_beginning;
global buffer_time;
global sphandle;
global MIN_EEG_MAX;
global scale_down_count;
global bargraph;
global eeg_max;
global enable;
global freq_matrix;
global data;
global count;
global PS3000_20MV PS3000_50MV PS3000_100MV PS3000_200MV PS3000_500MV ...
PS3000_1V PS3000_2V PS3000_5V PS3000_10V PS3000_20V PS3000_50V ...
PS3000_100V PS3000_200V PS3000_400V PS3000_RANGE PS3000_RANGE_MIN ...
PS3000_RANGE_MAX;
global PS3000_MAX_VALUE PS3000_MIN_VALUE PS3000_LOST_DATA;
global PS3000_TIMEBASE_MIN PS3000_TIMEBASE_MAX PS3000_TIMEBASE_BASE;
global BUFSIZE;
global OVERSAMPLE OVERSAMPLE_MIN OVERSAMPLE_MAX;
global autoscale; % envelope;
global first_plot;
global dc_coupled;
global use_webcam;
global max1 max2 max3 max4;
global thresh1 thresh2 thresh3 thresh4;
global fig2left fig2right fig2up fig2down;
global sample_time;
global left_t right_t up_t down_t;
global do_freq_analysis;
global fig3_posn;
global fig3_max_posn;
global fig3_bar_posn;
global himage;
global metricsx metricsy;
global channelsx channelsy;
global FREQ_BIN_SIZE;
global num_labels;
global bin_size_handle;
global label;
global old_sample_time old_BUFSIZE old_FREQ_BIN_SIZE;
global audio;
global last_tbuf;
global old_range;
global old_enable;
global old_couple;
global up_rect right_rect down_rect left_rect;
global last_freq_bin_size s;
global old_enable;
global old_bargraph;
global barhandle plothandle;
global old_buf;
global old_sample_time old_num_buffers old_bufsize;
global integration_time buffer_integration_time num_buffers required_samples buffer_samples;
global bufptr; % Next location for data (position of oldest data) ring buffer
global old_buf;
global old_sample_time old_num_buffers old_bufsize;
global fig1_x fig1_y fig1_width;
global show_metrics;
global recorded_count;
global screen_width screen_height;

% Formerly persistent variables (but persistent doesn't work right)
truncate_at_beginning = [];
buffer_time = [];
num_labels = [];
bin_size_handle = [];
label = [];
old_sample_time = [];
old_BUFSIZE = [];
old_FREQ_BIN_SIZE = [];
old_range = [];
old_enable = [];
old_couple = [];
up_rect = [];
right_rect = [];
down_rect = [];
left_rect = [];
last_freq_bin_size =[];
s = [];
old_enable = [];
old_bargraph = [];
barhandle = [];
plothandle = [];
old_buf = [];
old_sample_time = []; 
old_num_buffers = [];
old_bufsize= [];
integration_time = [];
buffer_integration_time = [];
num_buffers = [];
required_samples = [];
buffer_samples= [];
bufptr = [];
old_buf = [];
old_sample_time = [];
old_num_buffers = [];
old_bufsize= [];

recorded_count = 0;
show_metrics = 1;
sphandle = 9999;
MIN_EEG_MAX = 1e-10; % Because auto requires negative, and zero can't be negative
scale_down_count = 0;
bargraph = 0;
autoscale = 0;
FREQ_BIN_SIZE = 1; % Hz
eeg_max = -0.3; % default: -1 means auto, magnitude is max so far

% ampl env delta theta low_alpha high_alpha low_beta high_beta low_gamma high_gamma
metricsx = [1 0 0 0 0 0 0 0 0 0]; % Default: use amplitude
metricsy = [1 0 0 0 0 0 0 0 0 0]; % Default: use amplitude
% Default for h is unit 1, ch1-ch2, all others zero (channels down columns)
channelsx = [1 -1 0 0]';
% Default for vertical is unit 1 ch3-ch4, all others zero (channels down columns)
channelsy = [0 0 1 -1]';
freq_matrix = zeros(1,4,8); % Default, in case freq analysis not enabled

% Default enable all channels, max 4 units
% enable is num_pico (row) by 4 (channel, column)
enable = [1 1 1 1];
himage = 99999;
barhandle = 99999;
do_freq_analysis = 0;

left_t = 0;
right_t = 0;
up_t = 0;
down_t = 0;

max1 = 2;
max2 = 2;
max3 = 2;
max4 = 2;
thresh1 = 0.8*max1;
thresh2 = 0.8*max2;
thresh3 = 0.8*max3;
thresh4 = 0.8*max4;
dc_coupled = 0;
use_webcam = 0;
data.timebase = 10; % Default
data.InputRange=10;  % Default

% Size of data buffer
BUFSIZE=1024;

% Oversample: sample each point this many times (8 bits each)
OVERSAMPLE = 1;
OVERSAMPLE_MIN = 1;
OVERSAMPLE_MAX = 256;


if (use_picoscope)

	% loadlibrary won't work without mex properly setup; check if user needs to run mex -setup
	%    (right after install of Matlab)
	try 
		% This causes an error if mex is not set up
		a = mex.getCompilerConfigurations;
	catch err
	   	msgbox({'WARNING: mex compiler not set up yet.','Go to the Matlab command line, and type ''mex -setup'', choose a compiler, then try this again.'});
		fprintf('compiler not set up.\n');
	    return;
	end;

	% Suppress loadlibrary warnings; error in PicoTech .h file, can't fix
	warning off MATLAB:loadlibrary:TypeNotFound;
	warning off MATLAB:loadlibrary:TypeNotFoundForStructure;
	if libisloaded('PS3000')
	    msgbox({'PS3000.dll has already been loaded; close any running PicoMat RTI first.','If PicoMat RTI is not running, exit Matlab and restart.'});
		fprintf('could not load DLL.\n');
	    return;
	end
	try
		loadlibrary('PS3000.dll','PS3000.h');
		fprintf('PS3000.dll loaded');
	catch err
		msgbox({'ERROR: PS3000.dll could not be loaded.','Either it''s not found, or inappropriate for this machine.'});
		fprintf('could not load library.\n');
		return;
	end;
	if ~libisloaded('PS3000')
	    msgbox('library PS3000.dll or PS3000.h not found.') 
		return;
	end

	try
		data.num_pico = 0;
		opened = 1;
		while opened
			%-1 if the oscilloscope fails to open,
			%0 if no oscilloscope is found,
			%>0 (device handle) if the device opened
			fprintf('.');
			drawnow;
			data.unithandle(data.num_pico+1)=calllib('PS3000', 'ps3000_open_unit');
			if data.unithandle(data.num_pico+1) > 0
				opened = 1;
				%fprintf('Picoscope unit %d opened.\n',data.num_pico+1);
				data.num_pico = data.num_pico + 1;
			else
				opened = 0;
				%if data.unithandle(data.num_pico+1) == -1
					%fprintf('Found, but failed to open unit %d.\n',data.num_pico+1);
					%data.num_pico = data.num_pico + 1;
				%else % 0; no such oscilloscope
					%fprintf('Unit %d not found.\n',data.num_pico+1);
				%end;
			end;
			if data.num_pico >= max_pico
				% Stop opening units if max reached
				opened = 0;
				if max_pico > 1
					fprintf(' (limiting to %d units) ',max_pico);
				else
					fprintf(' (limiting to one unit) ');
				end;
			end;
		end; % while
		if data.num_pico == 0
			fprintf(' no PicoScope units found.\n');
	    	msgbox({'Sorry, no PicoScope units found.','If PicoMatlab RTI is not running, you should exit Matlab and restart.','This could be caused by an earlier crash of PicoMatlab RTI.'});
			unloadlibrary('PS3000');
			return;
		elseif data.unithandle(1) == -1
	    		msgbox('Found one or more PicoScope units, but failed to open one.') 
			unloadlibrary('PS3000');
			return;
		else
			% At least one good PicoScope
			fprintf('opened ');
			for i=1:data.num_pico
				s = get_picoscope_model(data.unithandle(i));
				fprintf('PicoScope %s',s);
				if i > 1
					fprintf(' (%d)',i);
				end;
				if data.num_pico > i
					%fprintf(', and ');
					fprintf(', ');
				else
					fprintf('.\n');
				end;
			end; % for
		end; %else
	catch err
	    msgbox('Error opening PicoScope unit.');
		for i=1:data.num_pico
			calllib('PS3000', 'ps3000_close_unit',data.unithandle(i));
		end;
		unloadlibrary('PS3000');
		return;
	end; %try/catch
	% Make enable match new num_pico size: default is 1x4, should be num_picox4
	enable(2:data.num_pico,1:4) = ones(data.num_pico-1,4); % Default: all channels enabled
	% Make channelsx,channelsy match new num_pico size: default is 4x1, should be 4xnum_pico
	channelsx(1:4,2:data.num_pico) = zeros(4,data.num_pico-1);
	channelsy(1:4,2:data.num_pico) = zeros(4,data.num_pico-1);
	% Similarly, frequency data is displayed on startup and needs to be init to right size.
	freq_matrix = zeros(data.num_pico,4,8); % Default, in case freq analysis not enabled
	
else
	% Not using picoscope
	data.num_pico = 0;
end; % if use_picoscope

% This destroys unithandle (in data), so must save and restore it (and a few others). 
% However, loading params after opening unit is better; only 1 dialog if it fails.
% Must do this even if not using picoscope, so these values don't get loaded!

save('.PMRTItemp.mat','data','enable','channelsx','channelsy'); % Save selected

load_settings(); % Tries to load save vars from disk; set all you can before, some necessary after

% Don't allow entire 'data' structure to be loaded; messes up EVERYTHING
new_data = data;
new_enable = enable;
new_channelsx = channelsx;
new_channelsy = channelsy;
load('.PMRTItemp.mat'); % Restore 'data' and other saved variables
delete('.PMRTItemp.mat'); % no mess

% Things to get from newdata:
%  (everything else needs to be EMPTY, as it is before load!!)
%   timebase
%   InputRange
data.timebase = new_data.timebase;
data.InputRange = new_data.InputRange;
% Join enable, channelsx, channelsy with existing (to avoid them being mismatched to data)
s1 = size(enable,1);
if s1 < size(new_enable,1) % saved enable has less channels than we currently do
	enable(1:s1,1:4) = new_enable(1:s1,1:4); % Replace part of data that was there
end;
s1 = size(channelsx,2);
if s1 < size(new_channelsx,2) % saved enable has less channels than we currently do
	channelsx(1:4,1:s1) = new_channelsx(1:4,1:s1); % Replace part of data that was there
end;
s1 = size(channelsy,2);
if s1 < size(new_channelsy,2) % saved enable has less channels than we currently do
	channelsx(1:4,1:s1) = new_channelsy(1:4,1:s1); % Replace part of data that was there
end;


sample_time = PS3000_TIMEBASE_BASE*2^(data.timebase);
count = 1;

% Possible voltage ranges for PS 3425
PS3000_20MV = 1; % Â±20 mV
PS3000_50MV = 2; % Â±50 mV
PS3000_100MV = 3;% Â±100 mV
PS3000_200MV = 4;% Â±200 mV
PS3000_500MV = 5;% Â±500 mV
PS3000_1V = 6;% Â±1 V
PS3000_2V = 7;% Â±2 V
PS3000_5V = 8;% Â±5 V
PS3000_10V = 9;% Â±10 V
PS3000_20V = 10;% Â±20 V
PS3000_50V = 11;% Â±50 V
PS3000_100V = 12; % Â±100 V
PS3000_200V = 13; %Â±200 V
PS3000_400V = 14;% Â±400 V
PS3000_RANGE = [20e-3 50e-3 100e-3 200e-3 500e-3 1 2 5 10 20 50 100 200 400];
PS3000_RANGE_MIN = 1;
PS3000_RANGE_MAX = 14;

% Raw data values
PS3000_MAX_VALUE= 32767;
PS3000_MIN_VALUE=-32767;
PS3000_LOST_DATA=-32768;

% Timebases; each step up doubles the time per sample.  
%  Calculated base empirically
PS3000_TIMEBASE_BASE = 5e-8; % Time for one sample at timebase 1, 
			        %  doubles from there every higher timebase
PS3000_TIMEBASE_MIN=2;
PS3000_TIMEBASE_MAX=19;

global fig2h fig2plot;
global fig3h;
global sep_off;
global window_padding;

fig2h = 99999;
fig2plot = 99999;
fig3h = 99999;
first_plot = 1;


% Get screen size; also need font sizes because difft between mac/pc
set(0,'units','pixels')
temp = get(0,'ScreenSize');
screen_width = temp(3);
screen_height = temp(4);
%fprintf('Screen size is %d by %d.\n',screen_width, screen_height);

sep_off = 10; % Separation of windows in multi-Picoscope mode
window_padding = 0; % 10 Windows are this much wider than what I specify

create_fig1_windows();

if (show_metrics)
	create_fig2_window();
end;

InputRange();
timebase();

if (do_freq_analysis)
	create_fig3_windows();
end; % do_freq

% Just set up first display
for unit_num=1:max(1,data.num_pico) % Do this once if not PicoScope mode, else however many PS
	SetChannelParams(unit_num,0,enable(unit_num,1),dc_coupled,data.InputRange,0,0,0);
	SetChannelParams(unit_num,1,enable(unit_num,2),dc_coupled,data.InputRange,0,0,0);
	SetChannelParams(unit_num,2,enable(unit_num,3),dc_coupled,data.InputRange,0,0,0);
	SetChannelParams(unit_num,3,enable(unit_num,4),dc_coupled,data.InputRange,0,0,0);
end;

sample_pico(); % Show initial data

function webcam_checkbox(a,b)
global data;
global use_webcam;
value=get(data.webcam_checkbox,'value');
if use_webcam == 0 && value == 1
	% Turning on webcam
	ans = questdlg('This will really control the lab webcam.  Do you want to continue?',...
			'Control webcam','Yes','Cancel','Cancel');
	if strcmp(ans,'Yes');
		use_webcam = 1;
	else
		use_webcam = 0;
		set(data.webcam_checkbox,'value',0);
	end;
else
	use_webcam= value;
end;

function bargraph_checkbox(a,b)
global data;
global bargraph;
value=get(data.bargraph_checkbox,'value');
bargraph = value;

function ch1_enable(calling_handle,calling_event)
% Must figure which unit this came from!  1 checkbox for every unit calls this callback
global data;
global enable;
found = 0;
for unit_num=1:max(1,data.num_pico)
	if calling_handle == data.ch1_checkbox(unit_num)
%fprintf('ch1_enable: found I was called from unit %d.\n',unit_num);
		value=get(data.ch1_checkbox(unit_num),'value');
		enable(unit_num,1) = value;
		found = 1;
	end; % if
end; % for
if ~found
	fprintf('ch1_enable: WARNING: did not find activating unit.\n');
end;

function ch2_enable(calling_handle,calling_event)
% Must figure which unit this came from!  1 checkbox for every unit calls this callback
global data;
global enable;
found = 0;
for unit_num=1:max(1,data.num_pico)
	if calling_handle == data.ch2_checkbox(unit_num)
%fprintf('ch2_enable: found I was called from unit %d.\n',unit_num);
		value=get(data.ch2_checkbox(unit_num),'value');
		enable(unit_num,2) = value;
		found = 1;
	end;
end;
if ~found
	fprintf('ch2_enable: WARNING: did not find activating unit.\n');
end;

function ch3_enable(calling_handle,calling_event)
% Must figure which unit this came from!  1 checkbox for every unit calls this callback
global data;
global enable;
found = 0;
for unit_num=1:max(1,data.num_pico)
	if calling_handle == data.ch3_checkbox(unit_num)
%fprintf('ch3_enable: found I was called from unit %d.\n',unit_num);
		value=get(data.ch3_checkbox(unit_num),'value');
		enable(unit_num,3) = value;
		found = 1;
	end;
end;
if ~found
	fprintf('ch3_enable: WARNING: did not find activating unit.\n');
end;

function ch4_enable(calling_handle,calling_event)
% Must figure which unit this came from!  1 checkbox for every unit calls this callback
global data;
global enable;
found = 0;
for unit_num=1:max(1,data.num_pico)
	if calling_handle == data.ch4_checkbox(unit_num)
%fprintf('ch4_enable: found I was called from unit %d.\n',unit_num);
		value=get(data.ch4_checkbox(unit_num),'value');
		enable(unit_num,4) = value;
		found = 1;
	end;
end;
if ~found
	fprintf('ch4_enable: WARNING: did not find activating unit.\n');
end;

function dc_checkbox(a,b)
global data;
global dc_coupled;
value=get(data.dc_checkbox,'value');
dc_coupled = value;

function freq_checkbox(a,b)
global data;
global do_freq_analysis;

value=get(data.freq_checkbox,'value');
if (~do_freq_analysis && value == 1)
	% Box just got checked
	create_fig3_windows();
	do_freq_analysis = 1;
end;
if (do_freq_analysis && value == 0)
	% Box just got unchecked
	delete_fig3_windows();
	do_freq_analysis = 0;
end; % if

function metric_checkbox(a,b)
global data;
global show_metrics;

value=get(data.metric_checkbox,'value');
if (~show_metrics && value == 1)
	% Box just got checked
	create_fig2_window();
	show_metrics = 1;
end;
if (show_metrics && value == 0)
	% Box just got unchecked
	delete_fig2_window();
	show_metrics = 0;
end; % if

function autoscale_checkbox(a,b)
global data;
global autoscale;
value=get(data.autoscale_checkbox,'value');
autoscale = value;
%fprintf('Autoscale set to %d (%f)\n',autoscale,value);

function start_sampling(a,b)
global do_sampling;
global use_picoscope;
do_sampling = 1;
while do_sampling && (~use_picoscope || libisloaded('PS3000'))
	drawnow; % Hopefully gives other callbacks a chance
	sample_pico();
end;

function stop_sampling(a,b)
global do_sampling;
do_sampling = 0;

function EXIT_prog(a,b)
global data;
global fig2h;
global use_picoscope;
global recorded_count;

% Warn if recorded data not saved
if recorded_count > 0
	ans = questdlg('You are exiting without saving your recorded data.  Do you really want to exit?',...
			'Exit without save?','Yes','Save and exit','Cancel','Cancel');
	if strcmp(ans,'Cancel');
		% Cancel, don't return
		return;
	elseif strcmp(ans,'Save and exit');
		save_recorded(0,0);
		% Now that we are saved, let exit proceed.
	else
		% Yes, exit. Do nothing, let routine proceed.
	end;
end;

stop_sampling();
if (use_picoscope)
	for i=1:data.num_pico
		calllib('PS3000', 'ps3000_stop',data.unithandle(i)); 
		calllib('PS3000', 'ps3000_close_unit',data.unithandle(i));
	end;
	unloadlibrary('PS3000');
	use_picoscope = 0;
end;
delete_fig1_windows();
delete_fig2_window();
delete_fig3_windows();

function sample_pico(a,b)
global data;
global PS3000_20MV PS3000_50MV PS3000_100MV PS3000_200MV PS3000_500MV ...
PS3000_1V PS3000_2V PS3000_5V PS3000_10V PS3000_20V PS3000_50V ...
PS3000_100V PS3000_200V PS3000_400V PS3000_RANGE PS3000_RANGE_MIN ...
PS3000_RANGE_MAX;
global PS3000_MAX_VALUE PS3000_MIN_VALUE PS3000_LOST_DATA;
global PS3000_TIMEBASE_MIN PS3000_TIMEBASE_MAX PS3000_TIMEBASE_BASE;
global BUFSIZE;
global OVERSAMPLE;
global auto0 auto1 auto2 auto3;
global autoscale;
global first_plot;
global use_picoscope;
global mode;
global input;
global count;
global dc_coupled;
global sample_time;
global do_freq_analysis;
global fig3h;
global enable;
global FREQ_BIN_SIZE;
global old_sample_time old_BUFSIZE old_FREQ_BIN_SIZE;
global audio;
global last_tbuf;
global show_metrics;
global buffer_time;
global truncate_at_beginning;


%	if ~myequal(old_sample_time,sample_time) || ~myequal(old_BUFSIZE,BUFSIZE) || ~myequal(old_FREQ_BIN_SIZE,FREQ_BIN_SIZE)
%		count = 1;
%	end;

	% Measure time active so far
	count = count + 1;
    	%sample_time = PS3000_TIMEBASE_BASE*2^(data.timebase);

	time = sample_time*BUFSIZE*count;
%fprintf('Simulated time: %f\n',time);

if (use_picoscope)
	% Gather pBuffers for ALL active units
	for unit_num=1:data.num_pico
%unit_num
%isfield(data,'unit')
%if isfield(data,'unit')
%	length(data.unit)
%end;
		if sum(enable(unit_num,:)) == 0
			data.unit(unit_num).pBuffer0.Value = zeros(1,BUFSIZE);
			data.unit(unit_num).pBuffer1.Value = zeros(1,BUFSIZE);
			data.unit(unit_num).pBuffer2.Value = zeros(1,BUFSIZE);
			data.unit(unit_num).pBuffer3.Value = zeros(1,BUFSIZE);
%fprintf('Unit %d channel all channels disabled.\n',unit_num);
			continue;
		end;

		% All channels not disabled
		if ~isfield(data,'unit') || length(data.unit) < unit_num 
			data.unit(unit_num).pBuffer0 = libpointer('int16Ptr',zeros(BUFSIZE,1));
			data.unit(unit_num).pBuffer1 = libpointer('int16Ptr',zeros(BUFSIZE,1));
			data.unit(unit_num).pBuffer2 = libpointer('int16Ptr',zeros(BUFSIZE,1));
			data.unit(unit_num).pBuffer3 = libpointer('int16Ptr',zeros(BUFSIZE,1));
			data.unit(unit_num).overflow = libpointer('int16Ptr',zeros(1,1));
			data.unit(unit_num).time_indisposed_ms = libpointer('int32Ptr',0);
		end; % if isempty

		drawnow; % Give other callbacks a chance
    		calllib('PS3000', 'ps3000_run_block',data.unithandle(unit_num),BUFSIZE,floor(data.timebase),OVERSAMPLE,data.unit(unit_num).time_indisposed_ms);
    		a = 0;
		timeout = 500; % 100 is too small
	    	while ~a && timeout > 0
       	 		a = calllib('PS3000','ps3000_ready',data.unithandle(unit_num));
%fprintf('Waiting for unit %d ready (%d).\n',unit_num,timeout);
%drawnow;
			timeout = timeout - 1;
    		end;
		if timeout <= 0
			fprintf('WARNING: PicoScope unit %d timed out, not ready.\n',unit_num);
			% Give dummy data, since unit didn't respond or is disabled
			data.unit(unit_num).pBuffer0.Value = zeros(1,BUFSIZE);
			data.unit(unit_num).pBuffer1.Value = zeros(1,BUFSIZE);
			data.unit(unit_num).pBuffer2.Value = zeros(1,BUFSIZE);
			data.unit(unit_num).pBuffer3.Value = zeros(1,BUFSIZE);
fprintf('Unit %d channel dummy data.\n',unit_num);
		else
    			a = calllib('PS3000', 'ps3000_get_values',data.unithandle(unit_num),data.unit(unit_num).pBuffer0,data.unit(unit_num).pBuffer1,data.unit(unit_num).pBuffer2,data.unit(unit_num).pBuffer3,data.unit(unit_num).overflow,BUFSIZE);
		end;

		drawnow; % Give other callbacks a chance

%fprintf('got values for unit %d (%d).\n',unit_num,a);
%data.unit(unit_num).pBuffer0.Value
%drawnow;

	end; % for unit_num
else % ~use_picoscope
	if mode == 1
		% Simulation random mode
		% Make dummy data, since no picoscope
		f = 1/5; % Hz
		arg = 2*pi*f*count*sample_time*BUFSIZE;
		ampl = 1.0;
		% Desired voltages
		v0 = enable(1,1)*rand(1,BUFSIZE)*ampl*sin(arg);
		v1 = enable(1,2)*rand(1,BUFSIZE)*ampl*sin(arg+pi/2);
		v2 = enable(1,3)*rand(1,BUFSIZE)*ampl*sin(arg+pi);
		v3 = enable(1,4)*rand(1,BUFSIZE)*ampl*sin(arg+3*pi/2);
		% Figure ADC counts that would result
		data.unit(1).pBuffer0.Value = PS3000_MAX_VALUE*v0/PS3000_RANGE(auto0(1));
		data.unit(1).pBuffer0.Value(data.unit(1).pBuffer0.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer0.Value(data.unit(1).pBuffer0.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer1.Value = PS3000_MAX_VALUE*v1/PS3000_RANGE(auto1(1));
		data.unit(1).pBuffer1.Value(data.unit(1).pBuffer1.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer1.Value(data.unit(1).pBuffer1.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer2.Value = PS3000_MAX_VALUE*v2/PS3000_RANGE(auto2(1));
		data.unit(1).pBuffer2.Value(data.unit(1).pBuffer2.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer2.Value(data.unit(1).pBuffer2.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer3.Value = PS3000_MAX_VALUE*v3/PS3000_RANGE(auto3(1));
		data.unit(1).pBuffer3.Value(data.unit(1).pBuffer3.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer3.Value(data.unit(1).pBuffer3.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		pause(BUFSIZE*sample_time);

		%fprintf('Count %d, sin(arg) = %f, max ch 0 is %f\n',count,sin(arg),max(data.unit(unit_num).pBuffer0(1).Value));
	elseif mode == 2
		% Simulation sweep mode
		% Change this frequency between 1/2 Hz and 35 Hz over 20 seconds, then sweep back, do it again
		use_mult = 1; % Use mult or add method
		sweep_time = 20; % seconds
		min_freq = 0.5; % Hz
		max_freq = 60; %60; % Hz
		sweep_counts = sweep_time/(BUFSIZE*sample_time); % Takes this many counts for one frequency sweep
%fprintf('%d counts in sweep from %f to %f.\n',sweep_counts,min_freq,max_freq);
		position = mod(count,sweep_counts);
		going_up = (mod(floor(count/sweep_counts),2) == 0);
		% Calculate frequency for this position, given that we are going up or down
		mult = exp(log(max_freq/min_freq)/sweep_counts);
		step = (max_freq-min_freq)/sweep_counts;
		if (going_up)
			if (use_mult)
				f = min_freq*mult^position; % Hz
				%posns = position:1/(BUFSIZE-1):position+1;
				%f = min_freq*mult.^posns; % Hz
			else
				f = min_freq+step*position; % Hz
			end;
		else
			if (use_mult)
				f = max_freq/mult^position; % Hz
				%posns = position:-1/(BUFSIZE-1):position-1;
				%f = max_freq./mult.^posns; % Hz
			else
				f = max_freq-step*position; % Hz
			end;
		end;
		if ishandle(fig3h(1))
			figure(fig3h(1));
			s = sprintf('Sweep frequency: %5.2f Hz',f(1));
			uicontrol('Style','text','String',s,'units','pixels','position',[260 -2 250 20],'FontSize',12,'ForegroundColor',[1 0.2 0.2],'BackgroundColor',[0 0 0]);
		end;
		t = time+(0:(BUFSIZE-1))*sample_time;
		arg = 2*pi*f.*t;
		ampl = 2.0; % volts
		% Desired voltages
		v0 = enable(1,1)*ampl*sin(arg);
%if exist('last_tbuf')
%	[count t(1)-last_tbuf sample_time]
%end;
%last_tbuf = t(BUFSIZE);
%if ~exist('last_tbuf') || isempty(last_tbuf)
%	last_tbuf = t;
%else
%	last_tbuf = [last_tbuf t];
%	figure(7);
%	plot(last_tbuf,'bo-');
%end;
		v1 = enable(1,2)*ampl*sin(arg+pi/2);
		v2 = enable(1,3)*ampl*sin(arg+pi);
		v3 = enable(1,4)*ampl*sin(arg+3*pi/2);

		% Figure ADC counts that would result
		data.unit(1).pBuffer0.Value = PS3000_MAX_VALUE*v0/PS3000_RANGE(auto0(1));
		data.unit(1).pBuffer0.Value(data.unit(1).pBuffer0.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer0.Value(data.unit(1).pBuffer0.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer1.Value = PS3000_MAX_VALUE*v1/PS3000_RANGE(auto1(1));
		data.unit(1).pBuffer1.Value(data.unit(1).pBuffer1.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer1.Value(data.unit(1).pBuffer1.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
	%if enable(1)
	%figure(4);		
	%subplot(2,1,1);
	%plot(v1);
	%subplot(2,1,2);
	%plot(data.unit(unit_num).pBuffer1(1).Value);
	%end;
		data.unit(1).pBuffer2.Value = PS3000_MAX_VALUE*v2/PS3000_RANGE(auto2(1));
		data.unit(1).pBuffer2.Value(data.unit(1).pBuffer2.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer2.Value(data.unit(1).pBuffer2.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer3.Value = PS3000_MAX_VALUE*v3/PS3000_RANGE(auto3(1));
		data.unit(1).pBuffer3.Value(data.unit(1).pBuffer3.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer3.Value(data.unit(1).pBuffer3.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		pause(BUFSIZE*sample_time);

	elseif mode == 3
		% Recorded data mode
		% Use data from input as fake real-time Picodata
		% Available: input.A, input.B, input.C, input.D, input.Tinterval, input.Length, input.Tstart

		% Figure voltages we'd expect to see on each channel
		time = count*sample_time*BUFSIZE; % Simulated real time, seconds
		data_length = input.Tinterval*input.Length; % Length of recorded data in seconds
		start_time = rem(time,data_length); % Start time of what we want to display in data array
		% The index below is always within the array.
		start_index = max(1,floor(start_time/input.Tinterval)); % Index into data to start
		% The index below may be out of the array.  Need to wrap data around.
		buffer_length = BUFSIZE*sample_time;
		end_index = floor((start_time+buffer_length)/input.Tinterval); % Index into data to end

		if isfield(input,'A')
			% Calculate channel 1 data
			v0 = enable(1,1)*index_and_resample(input.A,input.Length,input.Tinterval,sample_time,start_index,end_index);
		else
			v0 = zeros(1,BUFSIZE);
		end;
		if isfield(input,'B')
			% Calculate channel 2 data
			v1 = enable(1,2)*index_and_resample(input.B,input.Length,input.Tinterval,sample_time,start_index,end_index);
		else
			v1 = zeros(1,BUFSIZE);
		end;
		if isfield(input,'C')
			% Calculate channel 3 data
			v2 = enable(1,3)*index_and_resample(input.C,input.Length,input.Tinterval,sample_time,start_index,end_index);
		else
			v2 = zeros(1,BUFSIZE);
		end;
		if isfield(input,'D')
			% Calculate channel 4 data
			v3 = enable(1,4)*index_and_resample(input.D,input.Length,input.Tinterval,sample_time,start_index,end_index);
		else
			v3 = zeros(1,BUFSIZE);
		end;

		% Figure ADC counts that would result
		data.unit(1).pBuffer0.Value = PS3000_MAX_VALUE*v0/PS3000_RANGE(auto0(1));
		data.unit(1).pBuffer0.Value(data.unit(1).pBuffer0.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer0.Value(data.unit(1).pBuffer0.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer1.Value = PS3000_MAX_VALUE*v1/PS3000_RANGE(auto1(1));
		data.unit(1).pBuffer1.Value(data.unit(1).pBuffer1.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer1.Value(data.unit(1).pBuffer1.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer2.Value = PS3000_MAX_VALUE*v2/PS3000_RANGE(auto2(1));
		data.unit(1).pBuffer2.Value(data.unit(1).pBuffer2.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer2.Value(data.unit(1).pBuffer2.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer3.Value = PS3000_MAX_VALUE*v3/PS3000_RANGE(auto3(1));
		data.unit(1).pBuffer3.Value(data.unit(1).pBuffer3.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer3.Value(data.unit(1).pBuffer3.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		pause(BUFSIZE*sample_time);

	elseif mode == 4
		% Audio data mode. Try to get mono audio into v0 from microphone

		% Typically available: 8000, 11025, 22050, 44100, 48000, and 96000 Hz.
		% On Mac: must be between 1000 and 200000 inclusive
		sample_freq = 1/sample_time;
		sample_freq = max(1000,sample_freq); 
		sample_freq = min(200000,sample_freq); 
		if sample_freq ~= 1/sample_time
%fprintf('Requested sample rate is %f, can only do %f.\n',1/sample_time,sample_freq);
			sample_time = 1/sample_freq; % Alter sample time to something we can do
			data.timebase = log(sample_time/PS3000_TIMEBASE_BASE)/log(2); % Convert back to timebase
%fprintf('Sample time now %f, timebase %f.\n',sample_time,data.timebase);

			if ~myequal(old_sample_time,sample_time)
%fprintf('Setting and showing timebase.\n');
				set(data.guitimebase,'value',data.timebase);
				timebase(0,0);
			end; % if ~myequal
		end; % if sample

		bits = 16;
		channels = 2;
		new_audio_device = 0;
		if ~exist('audio') || isempty(audio)
			% Obvious buffer time (may alter later)
			buffer_time = BUFSIZE*sample_time;
			truncate_at_beginning = 1;
			% First iteration through audio (I hope)
			enable = [1 1 0 0];
			set(data.ch1_checkbox,'value',enable(1,1));
			set(data.ch2_checkbox,'value',enable(1,2));
			set(data.ch3_checkbox,'value',enable(1,3));
			set(data.ch4_checkbox,'value',enable(1,4));
			try
				audio = audiorecorder(sample_freq,bits,channels); % 16 bits, 2 channel (stereo)
				new_audio_device = 1;
			catch err
				msgbox({'Uh oh, there doesn''t seem to be any audio input.','That''s bad for audio mode.'});
				fprintf('no audio input.\n');
				EXIT_prog(0,0);
				return;
			end;
			fprintf('creating audio object, setting enables for audio.\n');
			%data.unit(unit_num).pBuffer1(1).Value = zeros(1,BUFSIZE); % Just in case
			data.unit(1).pBuffer2.Value = zeros(1,BUFSIZE);
			data.unit(1).pBuffer3.Value = zeros(1,BUFSIZE);
		else
			% Alter sample_freq if different
			if ~myequal(old_sample_time,sample_time)
				delete(audio);
%fprintf('Detected change in sample rate, remaking audio.\n');
				audio = audiorecorder(sample_freq,bits,channels); % 16 bits, 1 channel (mono)
				new_audio_device = 1;
				buffer_time = BUFSIZE*sample_time;
				truncate_at_beginning = 1;

			end;
		end;
		if (new_audio_device)
			% Sound devices sometimes take a while to get started.
			% Detect and remove zero data, alter buffer_time and truncate_at_beginning
		        % 	to avoid zero data
			recordblocking(audio,buffer_time);
			y = getaudiodata(audio);
			ZERO_THRESHOLD = 10e-5;
%figure(5);
%plot(y);
%pause;
			bad = (abs(y) < ZERO_THRESHOLD);
			for i=1:length(y)
				if bad(i)
					continue;			
				else
					break;
				end;
			end;
			truncate_at_beginning = i; % Lose this many samples to avoid zeros
			buffer_time = (BUFSIZE+i)*sample_time;
%fprintf('Calibration shows %d zero samples at beginning, truncating, extending buffer time to %f.\n',i,buffer_time);


		end; % if new audio

		recordblocking(audio,buffer_time);
		y = getaudiodata(audio);
		v0 = y(truncate_at_beginning:end,1)';
		v1 = y(truncate_at_beginning:end,2)';
		v0 = exact_size(v0,1,BUFSIZE);
		v1 = exact_size(v1,1,BUFSIZE);
%fprintf('After truncation, v0 has %d samples.\n',length(v0));
%size(v0)
%size(v1)

		% Figure ADC counts that would result
		data.unit(1).pBuffer0.Value = PS3000_MAX_VALUE*v0/PS3000_RANGE(auto0(1));
		data.unit(1).pBuffer0.Value(data.unit(1).pBuffer0.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer0.Value(data.unit(1).pBuffer0.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;
		data.unit(1).pBuffer1.Value = PS3000_MAX_VALUE*v1/PS3000_RANGE(auto1(1));
		data.unit(1).pBuffer1.Value(data.unit(1).pBuffer1.Value > PS3000_MAX_VALUE) = PS3000_MAX_VALUE;
		data.unit(1).pBuffer1.Value(data.unit(1).pBuffer1.Value < PS3000_MIN_VALUE) = PS3000_MIN_VALUE;

	end; % mode 4
end; % NOT use picoscope

% Here should have data.unit(unit_num).pBuffers for all active units, all channels, simulated or not

	drawnow; % Give other callbacks a chance
    samples = sample_time*(0:(BUFSIZE-1));
	% Backwards so unit 1 window always stays on top
	for unit_num=max(1,data.num_pico):-1:1 % Do this once if not PicoScope mode, else however many PS
    	result(unit_num,1,:) = PS3000_RANGE(auto0(unit_num))*double(data.unit(unit_num).pBuffer0.Value)/double(PS3000_MAX_VALUE);
	    result(unit_num,2,:) = PS3000_RANGE(auto1(unit_num))*double(data.unit(unit_num).pBuffer1.Value)/double(PS3000_MAX_VALUE);
   	 	result(unit_num,3,:) = PS3000_RANGE(auto2(unit_num))*double(data.unit(unit_num).pBuffer2.Value)/double(PS3000_MAX_VALUE);
   	 	result(unit_num,4,:) = PS3000_RANGE(auto3(unit_num))*double(data.unit(unit_num).pBuffer3.Value)/double(PS3000_MAX_VALUE);
    	auto0(unit_num) = SetChannelParams(unit_num,0,enable(unit_num,1),dc_coupled,auto0(unit_num),autoscale,min(data.unit(unit_num).pBuffer0.Value),max(data.unit(unit_num).pBuffer0.Value));
	    auto1(unit_num) = SetChannelParams(unit_num,1,enable(unit_num,2),dc_coupled,auto1(unit_num),autoscale,min(data.unit(unit_num).pBuffer1.Value),max(data.unit(unit_num).pBuffer1.Value));
   	 	auto2(unit_num) = SetChannelParams(unit_num,2,enable(unit_num,3),dc_coupled,auto2(unit_num),autoscale,min(data.unit(unit_num).pBuffer2.Value),max(data.unit(unit_num).pBuffer2.Value));
	    auto3(unit_num) = SetChannelParams(unit_num,3,enable(unit_num,4),dc_coupled,auto3(unit_num),autoscale,min(data.unit(unit_num).pBuffer3.Value),max(data.unit(unit_num).pBuffer3.Value));

		xpos = 0.1;
   	 	if (first_plot)
		    figure(data.fighandle(unit_num));
		    h = subplot(411);
		    data.h411(unit_num) = h;
		    set(h,'YLimMode','Auto');
			r = reshape(result(unit_num,1,:),1,BUFSIZE);
		    data.p411(unit_num) = plot(samples,r,'b');
			p = get(h,'Position');
			p(1) = xpos; % Move plot to the right
			set(h,'Position',p);
		    axis tight;
		    set(h,'YLimMode','Auto');
		    set(h,'XTick',[])
		    h = subplot(412);
		    data.h412(unit_num) = h;
			r = reshape(result(unit_num,2,:),1,BUFSIZE);
		    data.p412(unit_num) = plot(samples,r,'r');
			p = get(h,'Position');
			p(1) = xpos; % Move plot to the right
			set(h,'Position',p);
		    axis tight;
		    set(h,'YLimMode','Auto');
		    set(h,'XTick',[])
		    h = subplot(413);
		    data.h413(unit_num) = h;
			r = reshape(result(unit_num,3,:),1,BUFSIZE);
		    data.p413(unit_num) = plot(samples,r,'g');
			p = get(h,'Position');
			p(1) = xpos; % Move plot to the right
			set(h,'Position',p);
		    axis tight;
		    set(h,'YLimMode','Auto');
		    set(h,'XTick',[])
		    h = subplot(414);
		    data.h414(unit_num) = h;
			r = reshape(result(unit_num,3,:),1,BUFSIZE);
		    data.p414(unit_num) = plot(samples,r,'y');
			p = get(h,'Position');
			p(1) = xpos; % Move plot to the right
			set(h,'Position',p);
			ylabel('Signal (V)');
			xlabel('Time (s)');
		    axis tight;
		    set(h,'YLimMode','Auto');
		else
			r = reshape(result(unit_num,1,:),1,BUFSIZE);
   	 		set(data.p411(unit_num), 'YData',r);
			r = reshape(result(unit_num,2,:),1,BUFSIZE);
   	 		set(data.p412(unit_num), 'YData',r);
			r = reshape(result(unit_num,3,:),1,BUFSIZE);
   	 		set(data.p413(unit_num), 'YData',r);
			r = reshape(result(unit_num,4,:),1,BUFSIZE);
   	 		set(data.p414(unit_num), 'YData',r);
		end;
   	 %drawnow;
	end; % for unit_num
	first_plot = 0;

	if show_metrics
	    plot_metrics(samples,result);
	end;

	if do_freq_analysis
		for unit_num=1:max(1,data.num_pico) % Do this once if not PicoScope mode, else however many PS
%fprintf('buffering data at count %d, sample_time %f.\n',count,sample_time);
			[d1,d2,d3,d4] = buffer_data_to_required_integration_time2(unit_num,result(unit_num,1,:),result(unit_num,2,:),result(unit_num,3,:),result(unit_num,4,:));
%fprintf('doing freq analysis at count %d, sample_time %f.\n',count,sample_time);
			do_frequency_analysis(unit_num,d1,d2,d3,d4);
		end; % for
	end; % if do
    
	old_sample_time = sample_time;
	old_BUFSIZE = BUFSIZE;
	old_FREQ_BIN_SIZE = FREQ_BIN_SIZE;

	drawnow; % Give callbacks a chance

	% End of sample_pico

function timebase(a,b)
global data;
global PS3000_TIMEBASE_BASE;
global BUFSIZE;
global sample_time;
data.timebase=get(data.guitimebase,'value');
%data.timebase=floor(get(data.guitimebase,'value'));
sample_time = PS3000_TIMEBASE_BASE*2^(data.timebase);
%fprintf('Timebase %d: sample time is %f s, buffer will take %f s.\n',data.timebase,sample_time,sample_time*BUFSIZE);
sample_rate = 1/sample_time;
figure(data.fighandle(1));
uicontrol('style','text','Units','normalized', 'Position',[.21 0.0 0.21 0.03],'string',strcat('Rate: ',num2str(sample_rate, '%5.2e'),' Hz' ));

function InputRange(a,b)
global data;
global PS3000_RANGE;
global auto0 auto1 auto2 auto3;
for unit_num=1:max(1,data.num_pico) % Do this once if not PicoScope mode, else however many PS
	auto0(unit_num) = data.InputRange;
	auto1(unit_num) = data.InputRange;
	auto2(unit_num) = data.InputRange;
	auto3(unit_num) = data.InputRange;
end;
%fprintf('Range %d: range now +/- %f volts.\n',data.InputRange,PS3000_RANGE(data.InputRange));
%uicontrol('style','text','Units','normalized', 'Position',[.0 0.60 0.03 0.03],'string',num2str(data.InputRange));

% channel = 0,1,2,3
% Just sets DC coupling, range, doesn't set channel for acquisition!
% m and M are max and min of buffer in raw counts
function new_range = SetChannelParams(unitnum,channel,enable,dc_coupled,range,autoscale_data,m,M)
global data;
global PS3000_RANGE_MIN PS3000_RANGE_MAX;
global PS3000_MAX_VALUE PS3000_MIN_VALUE PS3000_LOST_DATA;
global use_picoscope;
global count;
global old_range;
global old_enable;
global old_couple;

new_range = range; % If no autoscale
%if (dc_coupled)
%	fprintf('Chan %d dc\n',channel);
%end;
if autoscale_data 
	if ( m == PS3000_MIN_VALUE || M == PS3000_MAX_VALUE )
		% Range is too small, auto scale up
		new_range = min(new_range+1,PS3000_RANGE_MAX);
		if (range ~= new_range)
			%if channel == 3
			%	fprintf('Channel %d: based on min %f and max %f, autoscaling up to larger range %d.\n',channel,m,M,new_range);
			%end;
		end;
	elseif ( m > 0.2*PS3000_MIN_VALUE &&  M < 0.2*PS3000_MAX_VALUE )
		% Range is too big, auto scale down
		new_range = max(new_range-1,PS3000_RANGE_MIN);
		if (range ~= new_range)
			%if channel == 3
			%	fprintf('Channel %d: based on min %f and max %f, autoscaling down to smaller range %d.\n',channel,m,M,new_range);
			%end;
		end; % if range
	end; % elseif
end; % autoscale_data

	
if (use_picoscope)
	% Only call set_channel lib function when enable, dc_coupled, or range has changed
	need_to_call =     ~exist('old_range(channel+1)') ...
			|| ~exist('old_enable(channel+1)') ...
			|| ~exist('old_couple(channel+1)');
	if ~need_to_call
		need_to_call =     ~myequal(old_range(channel+1),new_range) ...
				|| ~myequal(old_enable(channel+1),enable) ...
				|| ~myequal(old_couple(channel+1),dc_coupled);
	end; % if ~need
	if (need_to_call)
		a = calllib('PS3000', 'ps3000_set_channel',data.unithandle(unitnum),channel,enable,dc_coupled,new_range);
		a = 1; % Ignore error, nothing I can do. CMH 11/23/12
		if ~a
			fprintf('WARNING on count %d: setting channel %d, range %d, enable %d, ps3000_set_channel FAILED.\n',count, channel, new_range, enable);
			string_length = 50;
			init_val(1:string_length) = ' ';
			s = libpointer('stringPtr',init_val);
			code = 6; % 0 = version string, 6 = error code
			[num,str] = calllib('PS3000','ps3000_get_unit_info',data.unithandle(unitnum),s,string_length,code);
			fprintf('PicoScope error code (%d chars): ''%s''.\n',num,str);
		end; % if ~a
	end; % if need
end; % if use_picoscope

% Save old data before leaving
old_range(channel+1) = new_range;
old_enable(channel+1) = enable;
old_couple(channel+1) = dc_coupled;

function FlashLED(a,b)
global data;
global use_picoscope;
if (use_picoscope)
	calllib('PS3000', 'ps3000_flash_led',data.unithandle(1));
end;

function out = myrand(x,y)
	out = 2*(rand(x,y) - 0.5); % Between -1 and 1

% Figure metrics of data to use in control
function plot_metrics(samples,result)
	global fig2h fig2plot;
	global metricsx metricsy;
	global channelsx channelsy;
	global use_webcam;
	global max1 max2 max3 max4; 
	global thresh1 thresh2 thresh3 thresh4;
	global fig2left fig2right fig2up fig2down;
	global left_t right_t up_t down_t;
	global freq_matrix;
	global data;
	global up_rect right_rect down_rect left_rect;
	global count;
	global sample_time;
	global recorded_count;
	global recorded_time recorded_metricx recorded_metricy;
	global recorded_freq_matrix;
	global BUFSIZE;

	for unit_num=1:max(1,data.num_pico) % Do this once if not PicoScope mode, else however many PS
		for i=1:4
			% Max
			M(unit_num,i) = max(result(unit_num,i,:));
			% Min
			m(unit_num,i) = min(result(unit_num,i,:));
			% Envelope
			env(unit_num,i) = M(unit_num,i)-m(unit_num,i);
			% Amplitude
			ampl(unit_num,i) = pi*mean(abs(result(unit_num,i,:)));
		end;

		% ampl env delta theta low_alpha high_alpha low_beta high_beta low_gamma high_gamma
		metric_data(unit_num,1,:) = [ampl(unit_num,1) env(unit_num,1) reshape(freq_matrix(unit_num,1,1:8),1,8)];
		metric_data(unit_num,2,:) = [ampl(unit_num,2) env(unit_num,2) reshape(freq_matrix(unit_num,2,1:8),1,8)];
		metric_data(unit_num,3,:) = [ampl(unit_num,3) env(unit_num,3) reshape(freq_matrix(unit_num,3,1:8),1,8)];
		metric_data(unit_num,4,:) = [ampl(unit_num,4) env(unit_num,4) reshape(freq_matrix(unit_num,4,1:8),1,8)];

	end;

	% Choose metric data by channel and unit, then index; calculate SCALAR metricx, metricy
	% Matrices to use:
	%	channelsx (num_active_units by 4)
	% 	channelsy (num_active_units by 4)
	% 	metric_data (num_active_units by 4 by 10)
	%	metricsx (1 by 10)
	%	metricsy (1 by 10)

	% (num_active_units by 4 by 8) times (8 by 1)  =>  num_active_units by 4 by 1
	num_active_units = max(1,data.num_pico);
	% No matrix-way to do this, since 3-D.  Just brute force it.
	for i=1:size(metric_data,1)
		for j=1:size(metric_data,2)
			temp2 = reshape(metric_data(i,j,:),1,length(metricsx));
			tempx(i,j) = temp2 * metricsx';
			tempy(i,j) = temp2 * metricsy';
		end;
	end;
	% Multiply matrices el-by-el and sum
%tempx
%channelsx
%tempy
%channelsy
	metricx = sum(sum(tempx'.*channelsx));
	metricy = sum(sum(tempy'.*channelsy));

	recorded_count = recorded_count + 1;
	recorded_time(recorded_count) = sample_time*BUFSIZE*count;
	recorded_metricx(recorded_count) = metricx;
	recorded_metricy(recorded_count) = metricy;
	recorded_freq_matrix(recorded_count,:,:,:) = freq_matrix;

	if ishandle(fig2h)
    		set(fig2plot, 'XData',metricx);
	    	set(fig2plot, 'YData',metricy);
	end;

	% Do something when metrics go above threshold
	if metricx < -thresh2 && left_t == 0
		% Just entering threshold
		left_t = 1;
		%fprintf('Entered left threshold zone.\n');
		figure(fig2h);
		posn = [ -max2 -max4 max2-thresh2 max3+max4 ]; % x y w h
		left_rect = rectangle('Position',posn, 'EdgeColor',[1 0 0],'LineWidth',5);
		set(left_rect,'Visible','on');
		if (use_webcam)
			webcam_command('left');
		end;
	elseif metricx >= -thresh2 && left_t == 1
		%fprintf('Exited left threshold zone.\n');
		left_t = 0;
		figure(fig2h);
		set(left_rect,'Visible','off');
	end;

	if metricx > thresh1 && right_t == 0
		% Just entering threshold
		right_t = 1;
		%fprintf('Entered right threshold zone.\n');
		figure(fig2h);
		posn = [ thresh1 -max4 max1-thresh1 max3+max4 ]; % x y w h
		right_rect = rectangle('Position',posn, 'EdgeColor',[1 0 0],'LineWidth',5);
		set(right_rect,'Visible','on');
		if (use_webcam)
			webcam_command('right');
		end;
	elseif metricx <= thresh1 && right_t == 1
		%fprintf('Exited right threshold zone.\n');
		right_t = 0;
		figure(fig2h);
		set(right_rect,'Visible','off');
	end;

	if metricy > thresh3 && up_t == 0
		% Just entering threshold
		up_t = 1;
		%fprintf('Entered top threshold zone.\n');
		figure(fig2h);
		posn = [ -max2 thresh3 max2+max1 max3-thresh3 ]; % x y w h
		up_rect = rectangle('Position',posn, 'EdgeColor',[1 0 0],'LineWidth',5);
		set(up_rect,'Visible','on');
		if (use_webcam)
			webcam_command('up');
		end;
	elseif metricy <= thresh3 && up_t == 1
		%fprintf('Exited top threshold zone.\n');
		up_t = 0;
		figure(fig2h);
		set(up_rect,'Visible','off');
	end;

	if metricy < -thresh4 && down_t == 0
		% Just entering threshold
		down_t = 1;
		%fprintf('Entered bottom threshold zone.\n');
		figure(fig2h);
		posn = [ -max2 -max4 max2+max1 max4-thresh4 ]; % x y w h
		down_rect = rectangle('Position',posn, 'EdgeColor',[1 0 0],'LineWidth',5);
		set(down_rect,'Visible','on');
		if (use_webcam)
			webcam_command('down');
		end;
	elseif metricy >= -thresh4 && down_t == 1
		%fprintf('Exited bottom threshold zone.\n');
		down_t = 0;
		figure(fig2h);
		set(down_rect,'Visible','off');
	end;



	
function set_thresholds(a,b)

global thresh1 thresh2 thresh3 thresh4 ;
global fig2left fig2right fig2up fig2down;

prompt ={'Right','Left','Top','Bottom'};
title = 'Thresholds';
numlines = 1;
default = {num2str(thresh1),num2str(thresh2),num2str(thresh3),num2str(thresh4)};
resp = inputdlg_tight(prompt,title,numlines,default);
if ~isempty(resp)
	for i=1:4
		a = str2num(cell2mat(resp(i)));
		if ~isempty(a)
			val(i) = a;
		else
			val(i) = NaN;
		end;
	end;
	if ~isnan(val(1))
		thresh1 = val(1);
	end;
	if ~isnan(val(2))
		thresh2 = val(2);
	end;
	if ~isnan(val(3))
		thresh3 = val(3);
	end;
	if ~isnan(val(4))
		thresh4 = val(4);
	end;
end;

% Change visual thresholds
set(fig2left, 'XData',[-thresh2 -thresh2]);
set(fig2right, 'XData',[thresh1 thresh1]);
set(fig2up, 'YData',[-thresh4 -thresh4]);
set(fig2down, 'YData',[thresh3 thresh3]);


	
function set_maxes(a,b)

global max1 max2 max3 max4;
global fig2h;
global fig2left fig2right fig2up fig2down;

prompt ={'Right','Left','Top','Bottom'};
title = 'Maxima';
numlines = 1;
default = {num2str(max1),num2str(max2),num2str(max3),num2str(max4)};
resp = inputdlg_tight(prompt,title,numlines,default);
if ~isempty(resp)
	for i=1:4
		a = str2num(cell2mat(resp(i)));
		if ~isempty(a)
			val(i) = a;
		else
			val(i) = NaN;
		end;
	end;
	if ~isnan(val(1))
		max1 = val(1);
	end;
	if ~isnan(val(2))
		max2 = val(2);
	end;
	if ~isnan(val(3))
		max3 = val(3);
	end;
	if ~isnan(val(4))
		max4 = val(4);
	end;
end;

% Set max, min in each dimension
a = get(fig2h,'CurrentAxes');
set(a,'XLim',[-max2 max1]);
set(a,'YLim',[-max4 max3]);

set(fig2right, 'YData',[-max4 max3]);
set(fig2up, 'XData',[-max2 max1]);
set(fig2down, 'XData',[-max2 max1]);
set(fig2left, 'YData',[-max4 max3]);

function save_settings(a,b)

global show_metrics;
global bargraph;
global eeg_max;
global enable;
global metricsx metricsy;
global channelsx channelsy;
global data;
global BUFSIZE;
global OVERSAMPLE;
global autoscale; % envelope;
global dc_coupled;
global use_webcam;
global max1 max2 max3 max4;
global thresh1 thresh2 thresh3 thresh4;
global do_freq_analysis;
global FREQ_BIN_SIZE;

[name path success] = uiputfile('*.mat','Select a filename to store settings:');
if (success)
	filename = [path name];

	% Clear out data except for fields we want
	timebase = data.timebase;
	InputRange = data.InputRange;
	clear data;
	data.timebase = timebase;
	data.InputRange = InputRange;
	clear timebase InputRange;

	% Clear out stuff we don't want there
	clear a b path success;

	save(filename); % Save all global variables above (no others in current scope, I hope)
	s = sprintf('Parameters saved to %s.',name);
	uiwait(msgbox(s,'modal'));
end;

function load_settings()

global show_metrics;
global bargraph;
global eeg_max;
global enable;
global metricsx metricsy;
global channelsx channelsy;
global data;
global BUFSIZE;
global OVERSAMPLE;
global autoscale; % envelope;
global dc_coupled;
global use_webcam;
global max1 max2 max3 max4;
global thresh1 thresh2 thresh3 thresh4;
global do_freq_analysis;
global FREQ_BIN_SIZE;

[name path success] = uigetfile('*.mat','Select a filename to load settings:');
if (success)
	filename = [path name];
	load(filename); % Load all global variables above
	s = sprintf('Parameters loaded from %s.',name);
	uiwait(msgbox({s},'modal'));
end;

function set_params(a,b)
global BUFSIZE;
global OVERSAMPLE OVERSAMPLE_MIN OVERSAMPLE_MAX;
global data;
global sample_time;
global FREQ_BIN_SIZE;

prompt ={'Buffer size (samples)','Oversample (times)','Frequency bin size (Hz)'};
title = 'Acquisition';
numlines = 1;
default = {num2str(BUFSIZE),num2str(OVERSAMPLE),num2str(FREQ_BIN_SIZE)};
resp = inputdlg_tight(prompt,title,numlines,default);
if ~isempty(resp)
	for i=1:3
		a = str2num(cell2mat(resp(i)));
		if ~isempty(a)
			val(i) = a;
		else
			val(i) = NaN;
		end;
	end;
	if ~isnan(val(1))
		BUFSIZE = val(1);

		%fprintf('BUFSIZE set to %d.\n',BUFSIZE);
		% Change time data to match BUFSIZE 
    		samples = sample_time*(0:(BUFSIZE-1));
    		set(data.p411, 'XData',samples);
    		set(data.p412, 'XData',samples);
    		set(data.p413, 'XData',samples);
    		set(data.p414, 'XData',samples);
		% Fix axis limits
	    	set(data.h411,'XLim',[0 max(samples)]);
	    	set(data.h412,'XLim',[0 max(samples)]);
	    	set(data.h413,'XLim',[0 max(samples)]);
	    	set(data.h414,'XLim',[0 max(samples)]);
		% These last 4 only because Matlab complains if I wait
    		set(data.p411,'YData',zeros(1,BUFSIZE));
    		set(data.p412,'YData',zeros(1,BUFSIZE));
    		set(data.p413,'YData',zeros(1,BUFSIZE));
    		set(data.p414,'YData',zeros(1,BUFSIZE));
	end;
	if ~isnan(val(2))
		% Keep value in range
		temp = max(val(2),OVERSAMPLE_MIN);
		OVERSAMPLE = min(temp,OVERSAMPLE_MAX);
	end;
	if ~isnan(val(3))
		FREQ_BIN_SIZE = val(3);
	end;
end;

function set_eeg_max(a,b)
global eeg_max;
global MIN_EEG_MAX;

prompt ={'Max for display (or auto)'};
title = 'EEG max value';
numlines = 1;
if eeg_max < 0
	s = sprintf('Auto (%f)',-eeg_max);
	default = {s};
else
	default = {num2str(eeg_max)};
end;
resp = inputdlg_tight(prompt,title,numlines,default);
if ~isempty(resp)
	a = str2num(cell2mat(resp));
	if ~isempty(a)
		% If there's a pure number, I'll take it.
		eeg_max = max(MIN_EEG_MAX,a); % Don't let it be zero
	else
		% Otherwise, let eeg_max stay auto; neg sign, same magn
		eeg_max = -abs(eeg_max);
	end;
end;
%fprintf('Leaving set_eeg_max: value is %f.\n',eeg_max);

% String can be 'left', 'right', 'up', or 'down'
function webcam_command(string)
	switch(string)
		case 'left'
			fprintf('Webcam moving left.\n');
			[s,status] = special_urlread('http://nros415.com/hlab/webcam/move_left.php');
		case 'right'
			fprintf('Webcam moving right.\n');
			[s,status] = special_urlread('http://nros415.com/hlab/webcam/move_right.php');
		case 'up'
			fprintf('Webcam moving up.\n');
			[s,status] = special_urlread('http://nros415.com/hlab/webcam/move_up.php');
		case 'down'
			fprintf('Webcam moving down.\n');
			[s,status] = special_urlread('http://nros415.com/hlab/webcam/move_down.php');
	end;


function [s,status] = myurlread(incoming_url)

url = java.net.URL(incoming_url);
is = openStream(url);
isr = java.io.InputStreamReader(is);
br = java.io.BufferedReader(isr);
s = ' ';
while ~isempty(s)
	s = readLine(br);
	%disp(s);
end
s = '';
status = 1;

function do_frequency_analysis(unit_num,data1,data2,data3,data4)
	global freq_matrix;
	global fig3h;
	global sample_time;
	global himage;
	global FREQ_BIN_SIZE;
	global eeg_max;
	global enable;
	global bargraph;
	global fig3_posn;
	global scale_down_count;
	global data;
	global MIN_EEG_MAX;
	global fig3h;
	global num_labels;
	global bin_size_handle;
	global label;
	global last_freq_bin_size s;
	global old_enable;
	global old_bargraph;
	global barhandle plothandle;

	sample_freq = 1/sample_time;
	freq_bin_size = sample_freq / min([length(data1) length(data2) length(data3) length(data4)]);
%fprintf('d_f_a: data sizes: %d %d %d %d\n',length(data1),length(data2),length(data3),length(data4));
	%% Do it backwards so lowest-numbered window on top
	%for unit_num=max(1,data.num_pico):-1:1 % Do this once if not PicoScope mode, else however many PS
		% Don't waste time doing FFT of disabled channels
		if enable(unit_num,1)
			[ch1.a,ch1.b,ch1.c,ch1.d,ch1.e,ch1.f,ch1.g,ch1.h] = single_channel_eeg_analysis(data1,sample_freq);
		else
			[ch1.a,ch1.b,ch1.c,ch1.d,ch1.e,ch1.f,ch1.g,ch1.h] = deal(0);
		end;
		if enable(unit_num,2)
			[ch2.a,ch2.b,ch2.c,ch2.d,ch2.e,ch2.f,ch2.g,ch2.h] = single_channel_eeg_analysis(data2,sample_freq);
		else
			[ch2.a,ch2.b,ch2.c,ch2.d,ch2.e,ch2.f,ch2.g,ch2.h] = deal(0);
		end;
		if enable(unit_num,3)
			[ch3.a,ch3.b,ch3.c,ch3.d,ch3.e,ch3.f,ch3.g,ch3.h] = single_channel_eeg_analysis(data3,sample_freq);
		else
			[ch3.a,ch3.b,ch3.c,ch3.d,ch3.e,ch3.f,ch3.g,ch3.h] = deal(0);
		end;
		if enable(unit_num,4)
			[ch4.a,ch4.b,ch4.c,ch4.d,ch4.e,ch4.f,ch4.g,ch4.h] = single_channel_eeg_analysis(data4,sample_freq);
		else
			[ch4.a,ch4.b,ch4.c,ch4.d,ch4.e,ch4.f,ch4.g,ch4.h] = deal(0);
		end;

		matrix = vertcat([ch1.a ch1.b ch1.c ch1.d ch1.e ch1.f ch1.g ch1.h], ...
					 [ch2.a ch2.b ch2.c ch2.d ch2.e ch2.f ch2.g ch2.h], ...
					 [ch3.a ch3.b ch3.c ch3.d ch3.e ch3.f ch3.g ch3.h], ...
					 [ch4.a ch4.b ch4.c ch4.d ch4.e ch4.f ch4.g ch4.h]);
		freq_matrix(unit_num,1:4,1:8) = matrix;
%fprintf('Set freq_matrix for unit %d.\n',unit_num);

		num_enabled = sum(enable(unit_num,:));
		make_new_plot = 0;
		if ~bargraph && (length(himage) < unit_num || ~ishandle(himage(unit_num)))
			% Imagesc plot for this unit has never been created
			make_new_plot = make_new_plot + 1;
		end;
		if bargraph && (size(barhandle,1) < unit_num || ~ishandle(barhandle(unit_num,1)))
			% Bar graph for this unit has never been created
			make_new_plot = make_new_plot + 2;
		end;
		% Have not ever set old_enable or changed
		if size(old_enable,1) < unit_num || ~myequal(old_enable(unit_num,:),enable(unit_num,:))
			make_new_plot = make_new_plot + 3;
		end;
		% Bargraph settings changed
		if ~myequal(old_bargraph,bargraph)
			make_new_plot =  make_new_plot + 4;
		end;
		if make_new_plot
%fprintf('Making new plot (reason %d, unit %d).\n',make_new_plot,unit_num);
			figure(fig3h(unit_num));
			% Clear any existing 'uicontrol' channel labels
			if ~isempty(num_labels) && size(num_labels,2) >= unit_num && ~isempty(num_labels(unit_num))
%fprintf('num_labels for unit %d = %d, deleting old uicontrol labels.\n',unit_num,num_labels(unit_num));
				for i=1:num_labels(unit_num)
					if ~isempty(bin_size_handle) && ishandle(label(unit_num,i))
%fprintf('(unit %d) actually deleting uicontrol label %d: %f.\n',unit_num,i,label(unit_num,i));
						delete(label(unit_num,i));
					end;
				end;
			end;
	
			old_enable(unit_num,:) = enable(unit_num,:);
			old_bargraph = bargraph;
			if bargraph
				% Put bar graphs in the place where the imagesc was
				mycount = 1;
				for i=1:4
					if enable(unit_num,i)
						plothandle(unit_num,i) = subplot(num_enabled,1,mycount);
						barhandle(unit_num,i) = bar(1:8,matrix(i,:),0.9);
						%rotate(barhandle(i),[0 0 1],-54);
						set(gca,'XLim',[0.5 8.5]); % Very tight axes, no wasted space
						%set(barhandle(i),'ShowBaseLine','off');
						colormap(hot);
						caxis([0 1]); % abs(eeg_max)?
						set_bar_color_with_height(barhandle(unit_num,i));
						%set(plothandle(i),'YLim',[0 abs(eeg_max)]);
						posn = get(plothandle(unit_num,i),'Position');
						% Somehow, height is getting set to nearly zero. SHould be const.
						posn(4) = 1/num_enabled;
						set(plothandle(unit_num,i),'Position',posn);
						axis off;
						mycount = mycount + 1;
					end;
				end; % for
			else
				index = (1:4) .* enable(unit_num,:);
				index = index(index > 0);
				mat_plot = matrix(index,:);
				subplot(1,1,1); % Makes sure bar subplots go away
				himage(unit_num) = imagesc(mat_plot);
				colormap(hot);
				caxis([0 abs(eeg_max)]);
				%colorbar;
				axis off;
			end;
			x = 30;
			yoff = fig3_posn(4);
			ystep = -fig3_posn(4)/(num_enabled+1);
			w = 30;
			h = 15;
			mycount = 1;
			if enable(unit_num,1)
				s = 'Ch1';
				label(unit_num,mycount) = uicontrol('Style','text','String',s,'BackgroundColor',[0.8 0.8 0.8],'Position',[x mycount*ystep+yoff w h]);
%fprintf('setting uicontrol label(%d,%d) %f.\n',unit_num,mycount,label(unit_num,mycount));
				mycount = mycount + 1;
			end;
			if enable(unit_num,2)
				s = 'Ch2';
				label(unit_num,mycount) = uicontrol('Style','text','String',s,'BackgroundColor',[0.8 0.8 0.8],'Position',[x mycount*ystep+yoff w h]);
%fprintf('setting uicontrol label(%d,%d) %f.\n',unit_num,mycount,label(unit_num,mycount));
				mycount = mycount + 1;
			end;
			if enable(unit_num,3)
				s = 'Ch3';
				label(unit_num,mycount) = uicontrol('Style','text','String',s,'BackgroundColor',[0.8 0.8 0.8],'Position',[x mycount*ystep+yoff w h]);
%fprintf('setting uicontrol label(%d,%d) %f.\n',unit_num,mycount,label(unit_num,mycount));
				mycount = mycount + 1;
			end;
			if enable(unit_num,4)
				s = 'Ch4';
				label(unit_num,mycount) = uicontrol('Style','text','String',s,'BackgroundColor',[0.8 0.8 0.8],'Position',[x mycount*ystep+yoff w h]);
%fprintf('setting uicontrol label(%d,%d) %f.\n',unit_num,mycount,label(unit_num,mycount));
				mycount = mycount + 1;
			end;
			num_labels(unit_num) = mycount-1;
%fprintf('num labels for unit %d set to %d.\n',unit_num,num_labels(unit_num));
		
			y = fig3_posn(4)-9;
			xstep = fig3_posn(3)/10.2;
			xoff = 11; % 7
			w = xstep;
			h = 12; % 11
			for i=1:8
				s = cell2mat(freq_label(1,i));
				num_labels(unit_num) = num_labels(unit_num) + 1;
				label(unit_num,num_labels(unit_num)) = uicontrol('Style','text','String',s,'BackgroundColor',[0.8 0.8 0.8],'Position',[xoff+i*xstep y w h]);
			end;
%fprintf('num labels for unit %d set to %d.\n',unit_num,num_labels(unit_num));
			

		else % not make new plot
			% Autoscale first
			if eeg_max < 0
%fprintf('eeg_max is autoscaling.\n');
				% Autoscale EEG max up; data is bigger than max
				if abs(eeg_max) < max(max(matrix))
					eeg_max = -max(max(matrix))/0.9; % set eeg_max 11% (1/0.9) bigger than data
%fprintf('new higher eeg_max %f.\n',-eeg_max);
					scale_down_count = 0; % Reset count for scaling back down
				end;
				% Autoscale EEG max down; data is much smaller than max
				SCALE_DOWN_DELAY = 20;
				if max(max(matrix)) < 0.1*abs(eeg_max)
					scale_down_count = scale_down_count + 1;
%fprintf('scale_down_count = %d.\n',scale_down_count);
					if scale_down_count > SCALE_DOWN_DELAY
						eeg_max = -max(MIN_EEG_MAX, max(max(matrix))/0.9);	
						scale_down_count = 0; % Reset count for scaling back down
%fprintf('new lower eeg_max %f.\n',-eeg_max);
					end;
				else
					% Data is big enough for current range
					scale_down_count = 0;
				end;
			end;
			% Plot exists, just switch data
			if bargraph
				for i=1:4
					if enable(unit_num,i)
						set(barhandle(unit_num,i),'YData',matrix(i,:));
						set(plothandle(unit_num,i),'YLim',[0 abs(eeg_max)]);
						%set(plothandle(i),'YLimMode','auto');
						posn = get(plothandle(unit_num,i),'Position');
						% Somehow, height is getting set to nearly zero. SHould be const.
%fprintf('Plothandle(%d) position height is %f.\n',i,posn(4));
						posn(4) = 1/num_enabled;
						set(plothandle(unit_num,i),'Position',posn);
						%a = get(plothandle(unit_num),'CurrentAxes');
						%caxis([0 1]); 
						set(plothandle(unit_num),'CLim',[0 1]);
						set_bar_color_with_height(barhandle(unit_num,i));
					end;
				end;
			else
				% Assemble data only for enabled channels
				mycount = 1;
				temp = []; % In case nothing
				for i=1:4
					if enable(unit_num,i)
						temp(mycount,1:8) = matrix(i,:);
						mycount = mycount + 1;
					end;
				end;
				set(himage(unit_num),'CData',temp);
%fprintf('Unit %d: set ydata to temp.\n',unit_num);
				a = get(fig3h(unit_num),'CurrentAxes');
				%caxis([0 abs(eeg_max)]);
				set(a,'CLim',[0 abs(eeg_max)]);
			end;

		end;
%if isempty(last_freq_bin_size)
%fprintf('lfbs is empty.\n');
%else
%fprintf('lfbs is %f.\n',last_freq_bin_size);
%end;
		if unit_num == 1
			if isempty(last_freq_bin_size) || ~myequal(last_freq_bin_size,freq_bin_size)
				% Get rid of previous uicontrol text to avoid overwriting
				if ~isempty(bin_size_handle) && ishandle(bin_size_handle)
					delete(bin_size_handle);
				end;
				x = 10;
				y = 0;
				w = 170;
				h = 15;
				s = sprintf('Frequency bin size: %f Hz\n',freq_bin_size);
				figure(fig3h(1));
				bin_size_handle = uicontrol('Style','text','String',s,'BackgroundColor',[0.8 0.8 0.8],'Position',[x y w h]);
%fprintf('made bin size uicontrol.\n');
			end;
		end;
	%end; % for unit_num
%fprintf('fbs=%f\n',freq_bin_size);
	last_freq_bin_size = freq_bin_size;

function [a,b,c,d,e,f,g,h] = single_channel_eeg_analysis(in_data,sample_freq)
	global mode;

	if mode == 4 % audio mode
		% Analyze audio frequencies 10 Hz - 22KHz (based on 44.1KHz CD sample freq)
		% Synthesize 8 log-sized bins, get bigger as you go up
		freq_max = 22e3;
		freq_min = 10;
		mult = exp(log(freq_max/freq_min)/8);
		for i=1:8
			low_freq(i) = freq_min*mult^(i-1);
			high_freq(i) = freq_min*mult^(i);
		end;

	else
		% EEG frequencies

		delta_low_freq = 1; % All frequencies in Hz
		delta_high_freq = 3.5;
		theta_low_freq = 3.5;
		theta_high_freq = 7.5;
		low_alpha_low_freq = 7.5;
		low_alpha_high_freq = 9.5;
		high_alpha_low_freq = 9.5;
		high_alpha_high_freq = 12.5;
		low_beta_low_freq = 12.5;
		low_beta_high_freq = 17.5;
		high_beta_low_freq = 17.5;
		high_beta_high_freq = 30.5;
		low_gamma_low_freq = 30.5;
		low_gamma_high_freq = 40.5;
		high_gamma_low_freq = 40.5;
		high_gamma_high_freq = 50;
	
		low_freq = [delta_low_freq theta_low_freq low_alpha_low_freq high_alpha_low_freq...
					 low_beta_low_freq high_beta_low_freq low_gamma_low_freq high_gamma_low_freq];
		high_freq = [delta_high_freq theta_high_freq low_alpha_high_freq high_alpha_high_freq...
					 low_beta_high_freq high_beta_high_freq low_gamma_high_freq high_gamma_high_freq];
	end;

	bands = single_channel_freq_analysis(in_data,sample_freq,low_freq,high_freq);

	a = bands(1);
	b = bands(2);
	c = bands(3);
	d = bands(4);
	e = bands(5);
	f = bands(6);
	g = bands(7);
	h = bands(8);

function v = index_and_resample(A,Length,Tinterval,sample_time,start_index,end_index)
	global BUFSIZE;


		if (end_index > Length)
			data1 = A(start_index:Length);
			wrap_index = 1+mod(end_index,Length);
			% Wrap data around
			data2 = A(1:wrap_index);
			out_data = [data1' data2']';
		else
			out_data = A(start_index:end_index);
		end;
		% Now must resample time/data at sample_time of current array
		fsnew = 1/sample_time;
		fsold = 1/Tinterval;
		[p,q] = rat(fsnew/fsold);
		v = resample(out_data,p,q);

		% v is approximately right size, make it perfect
		if length(v) > BUFSIZE
			v = v(1:BUFSIZE); % Trim
		elseif length(v) < BUFSIZE
			v(length(v):BUFSIZE) = v(length(v));  % Duplicate last sample
		end;



% Commands to my webcam require retriving an URL inside the first url.
%  This allows me to hide nros415.webhost....(which can change) inside nros415.com (which doesn't);
function [s,status] = special_urlread(url)

	% Get data from original url
	[a,status] = urlread(url);
	http_locs = strfind(a,'http');
	quote_locs = strfind(a,'"');
	% Second http location is the one I want (first refers to some web org)
	start_loc = http_locs(2);
	% The quote loc I want is the next one after start
	temp = quote_locs(quote_locs > start_loc);
	end_loc = temp(1)-1;
	new_url = a(start_loc:end_loc);
	[s,status]  = urlread(new_url);

function set_metricsx(a,b)
global metricsx;

prompt ={'Amplitude','Envelope'};
for i=1:8
	prompt(i+2) = freq_label(2,i);
end;
title = 'Horizontal metrics';
numlines = 1;
% ampl env delta theta low_alpha high_alpha low_beta high_beta low_gamma high_gamma
default = {num2str(metricsx(1)), num2str(metricsx(2)), num2str(metricsx(3)), num2str(metricsx(4)), num2str(metricsx(5)), num2str(metricsx(6)), num2str(metricsx(7)), num2str(metricsx(8)), num2str(metricsx(9)), num2str(metricsx(10))};
resp = inputdlg_tight(prompt,title,numlines,default);
s = length(metricsx);
if ~isempty(resp)
	for i=1:s
		a = str2num(cell2mat(resp(i)));
		if ~isempty(a)
			val(i) = a;
		else
			val(i) = NaN;
		end;
	end;
	for i=1:s
		if ~isnan(val(i))
			metricsx(i) = val(i);
		end;
	end;
end;

function set_metricsy(a,b)
global metricsy;

prompt ={'Amplitude','Envelope'};
for i=1:8
	prompt(i+2) = freq_label(2,i);
end;
title = 'Vertical metrics';
numlines = 1;
% ampl env delta theta low_alpha high_alpha low_beta high_beta low_gamma high_gamma
default = {num2str(metricsy(1)), num2str(metricsy(2)), num2str(metricsy(3)), num2str(metricsy(4)), num2str(metricsy(5)), num2str(metricsy(6)), num2str(metricsy(7)), num2str(metricsy(8)), num2str(metricsy(9)), num2str(metricsy(10))};
resp = inputdlg_tight(prompt,title,numlines,default);
s = length(metricsy);
if ~isempty(resp)
	for i=1:s
		a = str2num(cell2mat(resp(i)));
		if ~isempty(a)
			val(i) = a;
		else
			val(i) = NaN;
		end;
	end;
	for i=1:s
		if ~isnan(val(i))
			metricsy(i) = val(i);
		end;
	end;
end;

function set_channelsx(a,b)
global channelsx;
global data;

prompt = {};
mycount = 1;
for unit_num=1:max(1,data.num_pico) % Do this once if not PicoScope mode, else however many PS
	for chan = 1:4
		if data.num_pico > 1
			s = sprintf('Unit %d channel %d',unit_num,chan);
		else
			s = sprintf('Channel %d',chan);
		end;
		prompt(mycount) = {s};
		mycount = mycount + 1;
	end;
end; % for unit
title = 'Horizontal channels';
numlines = 1;

% Assemble default from channelsx
mycount = 1;
default = {};
for unit_num=1:max(1,data.num_pico) 
	for chan = 1:4
		if chan <= size(channelsx,1) && unit_num <= size(channelsx,2)
			default(mycount) = {num2str(channelsx(chan,unit_num))};
		else
			default(mycount) = {'0'};
		end;
		mycount = mycount + 1;
	end;
end;

resp = inputdlg_tight(prompt,title,numlines,default);

s = length(resp);
if ~isempty(resp)
	for i=1:s
		unit_num = floor((i-1)/4) + 1;
		chan = mod((i-1),4)+1;
		a = str2num(cell2mat(resp(i)));
		if ~isempty(a)
			val(i) = a;
		else
			val(i) = NaN;
		end;
		if ~isnan(val(i))
			channelsx(chan,unit_num) = val(i);
%fprintf('set_channelsx: set channelsx(%d,%d) = %f.\n',chan,unit_num,val(i));
		end;
	end;
end;

function set_channelsy(a,b)
global channelsy;
global data;

prompt = {};
mycount = 1;
for unit_num=1:max(1,data.num_pico) % Do this once if not PicoScope mode, else however many PS
	for chan = 1:4
		if data.num_pico > 1
			s = sprintf('Unit %d channel %d',unit_num,chan);
		else
			s = sprintf('Channel %d',chan);
		end;
		prompt(mycount) = {s};
		mycount = mycount + 1;
	end;
end; % for unit
title = 'Vertical channels';
numlines = 1;

% Assemble default from channelsy
mycount = 1;
default = {};
for unit_num=1:max(1,data.num_pico) 
	for chan = 1:4
		if chan <= size(channelsy,1) && unit_num <= size(channelsy,2)
			default(mycount) = {num2str(channelsy(chan,unit_num))};
		else
			default(mycount) = {'0'};
		end;
		mycount = mycount + 1;
	end;
end;

resp = inputdlg_tight(prompt,title,numlines,default);

s = length(resp);
if ~isempty(resp)
	for i=1:s
		unit_num = floor((i-1)/4) + 1;
		chan = mod((i-1),4)+1;
		a = str2num(cell2mat(resp(i)));
		if ~isempty(a)
			val(i) = a;
		else
			val(i) = NaN;
		end;
		if ~isnan(val(i))
			channelsy(chan,unit_num) = val(i);
%fprintf('set_channelsy: set channelsy(%d,%d) = %f.\n',chan,unit_num,val(i));
		end;
	end;
end;

% Store past data and return enough to make required integration time
% b1,b2,b3,b4 are current buffers
% d1,d2,d3,d4 are buffers which are the proper integration time long
function [d1,d2,d3,d4] = buffer_data_to_required_integration_time2(unit_num,b1,b2,b3,b4)
	global FREQ_BIN_SIZE;
	global sample_time;
	global enable;
	global BUFSIZE;
	global data;
	global sphandle;
	global old_buf;
	global old_sample_time old_num_buffers old_bufsize;
	global integration_time buffer_integration_time num_buffers required_samples buffer_samples;
	global bufptr; % Next location for data (position of oldest data) ring buffer

	% Vectorize input data
	junk = reshape(b1(1,1,:),1,BUFSIZE);
	b(1,:) = junk;
	junk = reshape(b2(1,1,:),1,BUFSIZE);
	b(2,:) = junk;
	junk = reshape(b3(1,1,:),1,BUFSIZE);
	b(3,:) = junk;
	junk = reshape(b4(1,1,:),1,BUFSIZE);
	b(4,:) = junk;
%figure(6);
%plot(junk);
%pause;

	% Turn dummy on to see how long this routine takes
	dummy = 0;
	if (dummy)
		d1 = zeros(1,BUFSIZE);
		d2 = zeros(1,BUFSIZE);
		d3 = zeros(1,BUFSIZE);
		d4 = zeros(1,BUFSIZE);
		return;
	end;

	integration_time = 1/FREQ_BIN_SIZE;
	buffer_integration_time = length(b1)*sample_time;
	num_buffers = ceil(integration_time/buffer_integration_time);
	required_samples = floor(integration_time/sample_time);
	buffer_samples = BUFSIZE;
	num_active_units = max(1,data.num_pico);

	% Detect if some parameter has changed (FREQ_BIN_SIZE, sample_rate, BUFSIZE)
	% If so, clear old data (because may be invalid)
	if ~isempty(old_buf)
		% Have stored old buffers
		if ~myequal(old_sample_time,sample_time) || ~myequal(old_num_buffers,num_buffers) || ~myequal(old_bufsize,BUFSIZE)
%fprintf('Change in params detected, clearing old data.\n');
			% Preallocate big array
			old_buf = NaN*ones(num_active_units,num_buffers,4,BUFSIZE); % "empty" data is NaN
			bufptr(unit_num) = 0; % Gets incremented before use
		end;
	end;
%fprintf('old: %f %d %d.  new: %f %d %d.\n',old_sample_time, old_num_buffers,old_bufsize,...
%					  sample_time, num_buffers, BUFSIZE);
	old_sample_time = sample_time;
	old_num_buffers = num_buffers;
	old_bufsize = BUFSIZE;

%fprintf('Required freq bin size = %f Hz.\n',FREQ_BIN_SIZE);
%fprintf('Buffer freq bin size = %f Hz.\n',1/buffer_integration_time);
%fprintf('Required integration time = %f sec.\n',integration_time);
%fprintf('%d buffers required.\n',num_buffers);

	if (num_buffers < 2)
		% Integration time is within buffer, truncate it
		%index = 1:required_samples;
		% Integration time is within buffer, but don't truncate it (give better than required)
		index = 1:length(b1);
		d = b(:,index);
%fprintf('Integration time inside buffer, %d samples required out of %d (not truncating).\n',required_samples,length(b1));
	else
		% Integration time is longer than buffer must store old data to make this
%fprintf('Integration time bigger than buffer, need %d buffers.\n',num_buffers);
		if isempty(old_buf) 
			% Preallocate buffer for speed
			old_buf = NaN*ones(num_active_units,num_buffers,4,BUFSIZE); % "empty" data is NaN
			% bufptr may be empty too
			bufptr(unit_num) = 0; % Incremented before use
%fprintf('Preallocating buffer.\n');
		end;
		% Since we allocate old_buf as a block for all units, need to separately initialize bufptr
		if length(bufptr) < unit_num
			bufptr(unit_num) = 0; % Incremented before use
		end;
		bufptr(unit_num) = mod(bufptr(unit_num)-1+1,num_buffers)+1; % 1-based wraparound
		for chan=1:4
			if ~enable(unit_num,chan)
				% Don't bother storing anything for disabled channel
				d(chan,1:1) = 0; % Don't make it big because it will drive size of array
%fprintf('Making no buffers for disabled channel %d.\n',chan);
			else
				% Channel enabled
				% Put latest data into oldest position bufptr
				old_buf(unit_num,bufptr(unit_num),chan,1:BUFSIZE) = b(chan,:);
%fprintf('Channel %d: put data into old_buf(%d,%d,:).\n',chan,bufptr,chan);
				% Concatenate as much data as we need (or as we have)
				% Oldest entries have highest number, newest at 1
				% Empty data entries have NaN
				dtemp = [];
				for j=bufptr(unit_num)+1:bufptr(unit_num)+num_buffers
					i = mod(j-1,num_buffers)+1; % 1-based wraparound
					% If data is not empty
%fprintf('Channel %d: i=%d, old_buf(%d,%d,1) = %f.\n',chan,i,i,chan,old_buf(i,chan,1));
					if ~isnan(old_buf(unit_num,i,chan,1))
				 		temp = reshape(old_buf(unit_num,i,chan,:),1,BUFSIZE);
				 		dtemp = [dtemp temp];
%fprintf('Channel %d: buffer %d has good data.\n',chan,i);
					end;
				end;
%fprintf('Channel %d: data length %d samples.\n',chan,length(dtemp));
				if length(dtemp) > required_samples
					% Truncate exactly to required length
%fprintf('Channel %d has reached full size.\n',chan);
					d(chan,1:required_samples) = dtemp(1:required_samples);
				else
					d(chan,1:length(dtemp)) = dtemp;
%fprintf('channel %d data is %d samples out of %d required.\n',chan,length(d(chan,:)),required_samples);
				end;
			end; % else
		end; % for 
	end; % else

	% Create output data
	d1 = d(1,:);
	d2 = d(2,:);
	d3 = d(3,:);
	d4 = d(4,:);

% Show spectrum!
show_spectrum = 0;
if (show_spectrum)
if length(d1) == required_samples
	sample_freq = 1/sample_time;
	mag = abs(fftshift(fft(d1)));
	s = length(mag);
	freq = (-sample_freq/2):(sample_freq/(s-1)):(sample_freq/2);
	if ~ishandle(sphandle)
		figure('name','Spectrum','Position',[645    28   560   183]);
		%plot(d1,'bo-');
		sphandle = plot(freq,mag);
		axis([-60 60 0 max(mag)]);
	else
		set(sphandle,'YData',mag);
	end;
end;
end;

% Return 1 if a is equal to b, even if one is empty.
% Seems like the obvious choice, but Matlab doesn't do it.
function o = myequal(a,b)
	if isempty(a) && isempty(b)
		o = 1;
	elseif isempty(a) && ~isempty(b)
		o = 0;
	elseif isempty(b) && ~isempty(a)
		o = 0;
	else 
		o = (a == b);
		% Unvectorize: all elements must be equal to return 1; otherwise zero
		if sum(o) < length(a)
			o = 0;
		else
			o = 1;
		end;
	end;

function bands = single_channel_freq_analysis(in_data,sample_freq,low_freq,high_freq)

	in_data = in_data - mean(in_data); % Ignore DC, just in case
	mag = abs(fftshift(fft(in_data)));
%figure(5);
%subplot(2,1,1);
%plot(in_data);
%subplot(2,1,2);
%plot(mag);
%pause;
	s = length(mag);
	mag = 2*mag/s; % Normalize by half FFT size to get coefficients correct
	freq = (-sample_freq/2):(sample_freq/(s-1)):(sample_freq/2);
%sample_freq
%low_freq
%high_freq

	for i=1:length(low_freq)
		lf = low_freq(i); hf = high_freq(i);
		region = (freq >= lf) & (freq <= hf);
		bands(i) = sum(mag(region)); % should be sum or mean? sum right, but mean looks better
	end;

function s = freq_label(index,type)
global mode;

if mode == 4 % audio data

	% Audio bands
	label(1,1) = {'Low bass'};
	label(1,2) = {'High bass'};
	label(1,3) = {'Bass/Mid'};
	label(1,4) = {'Low mid'};
	label(1,5) = {'High mid'};
	label(1,6) = {'Mid/Treble'};
	label(1,7) = {'Treble'};
	label(1,8) = {'High Treble'};

	label(2,1) = {'Low bass'};
	label(2,2) = {'High bass'};
	label(2,3) = {'Bass/Mid'};
	label(2,4) = {'Low midrange'};
	label(2,5) = {'High midrange'};
	label(2,6) = {'Mid/Treble'};
	label(2,7) = {'Treble'};
	label(2,8) = {'High Treble'};


else

	% EEG bands
	label(1,1) = {'Delta'};
	label(1,2) = {'Theta'};
	label(1,3) = {'alpha'};
	label(1,4) = {'Alpha'};
	label(1,5) = {'beta'};
	label(1,6) = {'Beta'};
	label(1,7) = {'gamma'};
	label(1,8) = {'Gamma'};

	label(2,1) = {'Delta band'};
	label(2,2) = {'Theta band'};
	label(2,3) = {'Low alpha'};
	label(2,4) = {'High alpha'};
	label(2,5) = {'Low beta'};
	label(2,6) = {'High beta'};
	label(2,7) = {'Low gamma'};
	label(2,8) = {'High gamma'};

end;

s = label(index,type);


% Set the colormap and caxis first
function set_bar_color_with_height(bar_handle)
	global eeg_max;

	ch = get(bar_handle,'Children');
	fvd = get(ch,'Faces');
	fvcd = get(ch,'FaceVertexCData');
	y = get(bar_handle,'YData')';
	n = length(y);
	% Set color of each bar according to height of y
	for i = 1:n
   	 	fvcd(fvd(i,:)) = y(i)/abs(eeg_max);
	end
	set(ch,'FaceVertexCData',fvcd)

function stop_pico()
	global data;
	calllib('PS3000', 'ps3000_stop',data.unithandle(1)); 
	
function close_and_reopen_unit()
	global data;
	calllib('PS3000', 'ps3000_close_unit',data.unithandle(1)); 
	data.unithandle(1)=calllib('PS3000', 'ps3000_open_unit');

function s = get_picoscope_model(handle)
	string_length = 50;
	init_val(1:string_length) = ' ';
	sptr = libpointer('stringPtr',init_val);
	code = 3; % 0 = version string, 3 = PS model, 6 = error code
	[num,str] = calllib('PS3000','ps3000_get_unit_info',handle,sptr,string_length,code);
	s = str;

function create_fig3_windows()
	global data;
	global fig3_posn;
	global fig3_max_posn;
	global fig3_bar_posn;
	global bargraph;
	global fig3h;
	global screen_width screen_height;
	global sep_off;
	global window_padding;
	global fig1_x fig1_y;
	global fig1_width;

	% Position window for high PicoScope counts
	fig3_width = 560;
	fig3_height = 175;
	fig3_y = 47;
	fig3_x = 49;
	fig3_posn = [  fig3_x(1)  fig3_y   fig3_width(1)  fig3_height ]; % default posn
	if data.num_pico <= 1
		% Put single fig3 normal position
	else
		% Match width and posn with figure 1
		fig3_width = fig1_width;
		fig3_x = fig1_x;
	end;
	fig3_max_posn = [fig3_width(1)-50 fig3_height-50 40 30];
	fig3_bar_posn = fig3_max_posn;
	fig3_bar_posn(2) = fig3_bar_posn(2) - 30;

	% Go backwards so lower-numbered channel window on top!
	for unit_num=max(1,data.num_pico):-1:1 % Do this once if not PicoScope mode, else however many PS

		x = fig3_x(unit_num);
		y = fig3_y;
		w = fig3_width(unit_num);
		h = fig3_height;
		posn = [x y w h];
		if data.num_pico > 1
			s = sprintf('Spectrum analysis window %d',unit_num);
		else
			s = sprintf('Spectrum analysis window');
		end;
	
		fig3h(unit_num) = figure('Name',s,'Position', posn,'CloseRequestFcn',@EXIT_prog);
	
		if unit_num == 1
			uicontrol('Style', 'pushbutton', 'String', 'Max',...
		    	'Position', fig3_max_posn, 'Callback', @set_eeg_max);
			data.bargraph_checkbox = uicontrol('style','checkbox','units','pixels',...
   		             'position',fig3_bar_posn,'String','Bar','value',bargraph,'Callback',@bargraph_checkbox);
		end; % if unit_num
	end; % for

function delete_fig3_windows()
	global data;
	global fig3h;

	if isfield(data,'num_pico')
		for unit_num=1:max(1,data.num_pico) % Do this once if not PicoScope mode, else however many PS
			if length(fig3h) >= unit_num
				if ishandle(fig3h(unit_num))
					delete(fig3h(unit_num));
				end;
			end;
		end; % for
	end;
	fig3h = 99999;

function create_fig2_window()
	global data;
	global fig2h fig2plot;
	global fig2left fig2right fig2up fig2down;
	global fig1_x fig1_y;
	global max1 max2 max3 max4;
	global thresh1 thresh2 thresh3 thresh4;
	global use_webcam;
	global screen_width screen_height;
	global sep_off;
	global window_padding;

	% Position window for high PicoScope counts
	if data.num_pico <= 1
		% Full width
		fig2_width = 560;
		% Put fig2 next to fig1
		fig2_x = fig1_x + 581; 
	else
		% Split width among num_pico+1 windows
		sep = (sep_off+window_padding)*(data.num_pico+1);
		fig2_width = (screen_width-sep)/(data.num_pico+1);
		% Put fig2 at far right side of screen
		fig2_x = screen_width - (fig2_width+sep_off); 
	end;

	fig2_height = 420;
	fig2h=figure('Name','Metric window','Position', [ fig2_x fig1_y   fig2_width   fig2_height ],'CloseRequestFcn',@EXIT_prog);
	hold off;
	fig2plot = plot(0,0,'ro');
	xlabel('Horizontal metric');
	ylabel('Vertical metric');
	axis([-max2 max1 -max4 max3]);
	hold on;
	fig2left = plot([-thresh2 -thresh2],[-max4 max3],'b-');
	fig2right = plot([thresh1 thresh1],[-max4 max3],'b-');
	fig2up = plot([-max2 max1],[-thresh4 -thresh4],'b-');
	fig2down = plot([-max2 max1],[thresh3 thresh3],'b-');
	hold off;

	xoff = 5; % 20
	xstep = 70; % 80
	y = fig2_height-30;
	w = 70; % 80
	h = 30;
	uicontrol('Style', 'pushbutton', 'String', 'Max values',...
	    'Position', [xoff y w h], 'Callback', @set_maxes);
	uicontrol('Style', 'pushbutton', 'String', 'Thresholds',...
	    'Position', [xoff+xstep y w h], 'Callback', @set_thresholds);
	uicontrol('Style', 'pushbutton', 'String', 'H Metrics',...
	    'Position', [xoff+2.1*xstep y w h], 'Callback', @set_metricsx);
	uicontrol('Style', 'pushbutton', 'String', 'V Metrics',...
	    'Position', [xoff+3.1*xstep y w h], 'Callback', @set_metricsy);
	uicontrol('Style', 'pushbutton', 'String', 'H Channels',...
	    'Position', [xoff+4.2*xstep y w h], 'Callback', @set_channelsx);
	uicontrol('Style', 'pushbutton', 'String', 'V Channels',...
	    'Position', [xoff+5.2*xstep y w h], 'Callback', @set_channelsy);
	fs = 15; % 15 too big for PC
	data.webcam_checkbox = uicontrol('style','checkbox','units','pixels',...
	                'position',[fig2_width-90,fig2_height-25,140,20],'String','Webcam',...
			'value',use_webcam,'Callback',@webcam_checkbox, ...
   	      'FontSize',13,'ForegroundColor',[1 0.2 0.2],'BackgroundColor',[0 0 0]);
	uicontrol('Style', 'pushbutton', 'String', 'Save recorded data',...
	    'Position', [xoff+5.2*xstep 5 130 30], 'Callback', @save_recorded); % 5.6

function delete_fig2_window()
	global fig2h;

	% There can be only one.   
	if ishandle(fig2h)
		delete(fig2h); % Should delete all children, too.
	end;
	fig2h = 9999;

function delete_fig1_windows()
	global data;

	if isfield(data,'num_pico') && isfield(data,'fighandle')
		for unit_num=1:max(1,data.num_pico) % Do this once if not PicoScope mode, else however many PS
			if length(data.fighandle) >= unit_num && ishandle(data.fighandle(unit_num))
				delete(data.fighandle(unit_num));
				data.fighandle(unit_num) = 9999;
			end;
		end;
	end;

% Save everything in settings file, plus data (data can be loaded as settings)
function save_recorded(a,b)

	% These from save_settings, must be saved here, too
	global show_metrics;
	global bargraph;
	global eeg_max;
	global enable;
	global metricsx metricsy;
	global channelsx channelsy;
	global data; % just need timebase, InputRange
	global BUFSIZE;
	global OVERSAMPLE;
	global autoscale; % envelope;
	global dc_coupled;
	global use_webcam;
	global max1 max2 max3 max4;
	global thresh1 thresh2 thresh3 thresh4;
	global do_freq_analysis;
	global FREQ_BIN_SIZE;
	% These are only for saved data, not included in settings
	global recorded_count;
	global recorded_time recorded_metricx recorded_metricy;
	global sample_time; % Save this although it's redundant
	global recorded_freq_matrix;

	[name path success] = uiputfile('*.mat','Select a filename to store recorded data:');
	if (success)
%figure(5);
%plot(recorded_time,recorded_metricx,'bo-');
		filename = [path name];
		Tstart = min(recorded_time);
		Tinterval = sample_time*BUFSIZE;
		% Put NaNs in data where time skips to keep timescale accurate
		A = recorded_metricx(1);
		B = recorded_metricy(1);
		adjusted_time = Tstart;
		for i=2:length(recorded_time)
			dif = recorded_time(i)-recorded_time(i-1);
			if (dif > Tinterval)
				% Need to put in some dummy points
				num = floor(dif/Tinterval);
				A = [A NaN*ones(1,num-1) recorded_metricx(i)];
				B = [B NaN*ones(1,num-1) recorded_metricy(i)];
				adjusted_time = [adjusted_time (recorded_time(i-1)+Tinterval):Tinterval:recorded_time(i)];
			else
				% Just add the point
				A = [A recorded_metricx(i)];
				B = [B recorded_metricy(i)];
				adjusted_time = [adjusted_time recorded_time(i)];
			end;
		end;
		% Replace data with "filled out" data (with NaNs)
		recorded_metricx = A;
		recorded_metricy = B;
		recorded_time= adjusted_time;
	
		% Clear out data except for fields we want
		timebase = data.timebase;
		InputRange = data.InputRange;
		clear data;
		data.timebase = timebase;
		data.InputRange = InputRange;

		% Get rid of some stuff
		clear A B adjusted_time a b path success;
		clear i dif Tinterval Tstart num;

		% Save all local and global variables in current scope.
		save(filename);
	
		% Clear recorded data to avoid saving it twice
		recorded_count = 0;
		recorded_time = [];
		recorded_metricx = [];
		recorded_metricy = [];
		recorded_freq_matrix = [];

		s = sprintf('Data saved to %s.',name);
		uiwait(msgbox(s,'modal'));

	end;

function create_fig1_windows()
	global data;
	global fig1_x fig1_y;
	global program_name;
	global modename;
	global autoscale; 
	global enable; 
	global dc_coupled show_metrics do_freq_analysis;
	global PS3000_TIMEBASE_MIN PS3000_TIMEBASE_MAX;
	global screen_width screen_height;
	global sep_off;
	global window_padding;
	global fig1_width;

	% Position window for high PicoScope counts
	first_window_min_width = 500; % Less than this, controls don't work
	if data.num_pico <= 1
		% Full width
		fig1_width(1) = 560;
		% Put single fig1 normal position
		fig1_x(1) = 48;
	else
		% Split width among num_pico+1 windows (including fig2)
		sep = (sep_off+window_padding)*(data.num_pico+1);
		fig2_width = (screen_width-sep)/(data.num_pico+1); % width of fig2 window

		fig1_width(1) = max(first_window_min_width,fig2_width);
		fig1_x(1) = sep_off;
		fig1_rest_width = (screen_width-fig1_width(1)-fig2_width-sep)/(data.num_pico-1);
		fig1_width(2:data.num_pico) = fig1_rest_width;
		index = 2:data.num_pico;
		fig1_x(index) = 2*sep_off+fig1_width(1)+(index-2)*(sep_off+fig1_rest_width);
	end;

	% Create figure 1
	% Go backwards so lower-numbered channel window on top!
	fig1_height = 420;
	fig1_y = 288;
	for win_num=max(1,data.num_pico):-1:1  % Create one window if not PicoScope mode, else however many PS
		% Create main window
		if data.num_pico > 1 
			s = sprintf('Raw data window %d',win_num);
		else
			s = sprintf('Raw data window');
		end;
		data.fighandle(win_num)=figure('Name',s,'CloseRequestFcn',@EXIT_prog);
		set(data.fighandle(win_num),'Position', [ fig1_x(win_num) fig1_y fig1_width(win_num) fig1_height]);
	
		%size1 = 17;
		%size2 = 14;
		size1 = 12;
		size2 = 10;
	
		if win_num == 1
			% Put program name in window 1 only
			uicontrol('Style','text','String',program_name,'units','pixels','position',[260 15 250 20],'FontSize',size1,'ForegroundColor',[1 0.2 0.2],'BackgroundColor',[0 0 0]); % 17 was good, 10 better on XP
			uicontrol('Style','text','String',modename,'units','pixels','position',[260 -2 250 20],'FontSize',size2,'ForegroundColor',[1 0.2 0.2],'BackgroundColor',[0 0 0]); % 14 was good, 8 better on XP
		end;
	
		if win_num == 1
			% Start, stop, and exit only on main raw data window
			y = 7; % 15
			fs = 13; % 17 too big for windows, looks fine on mac
			uicontrol('Style', 'pushbutton', 'String', 'Start',...
   			 'Position', [5 y 40 30], 'Callback', @start_sampling);
			uicontrol('Style', 'pushbutton', 'String', 'Stop',...
   			 'Position', [45 y 40 30], 'Callback', @stop_sampling);
			uicontrol('Style', 'pushbutton', 'String', 'EXIT',...
			    'Position', [fig1_width(1)-47 y 45 30], 'Callback', @EXIT_prog,...
   			  'FontSize',fs);%,'ForegroundColor',[1 0.0 0.0],'BackgroundColor',[0 0 0]);
	
			% Autoscale, dc_coupled, do_freq only on main window, apply for all
			x = 5; % 20
			y = fig1_height-20;
			data.autoscale_checkbox = uicontrol('style','checkbox','units','pixels',...
   	             'position',[x,y,80,20],'String','Autorange',...
				     'value',autoscale,'Callback',@autoscale_checkbox);
			x = 90; % 110
			data.dc_checkbox = uicontrol('style','checkbox','units','pixels',...
		                'position',[x,y,80,20],'String','DC coupled',...
						'value',dc_coupled,'Callback',@dc_checkbox);
			x = fig1_width(1)-180;
			data.freq_checkbox = uicontrol('style','checkbox','units','pixels',...
		                'position',[x,y,100,20],'String','Freq analysis',...
						'value',do_freq_analysis,'Callback',@freq_checkbox);
			x = fig1_width(1)-90;
			data.metric_checkbox = uicontrol('style','checkbox','units','pixels',...
		                'position',[x,y,100,20],'String','Show metrics',...
						'value',show_metrics,'Callback',@metric_checkbox);
			% Timebase only on main window, applies for all
			data.guitimebase=uicontrol('style','slider',...
					'value',data.timebase,...
					'min',PS3000_TIMEBASE_MIN,'max',PS3000_TIMEBASE_MAX+0.1,...
					'SliderStep',[1/19 1/19],...
					'Position', [120 10 120 20],'String','text',...
					'callback',@timebase); 
			% Acquisition params, save only on main window
			x = 165; % 200
			y = fig1_height-30;
			uicontrol('Style', 'pushbutton', 'String', 'Acquisition',...
			    'Position', [x y 80 30], 'Callback', @set_params);
			x = 245; % 280
			uicontrol('Style', 'pushbutton', 'String', 'Save settings',...
   			    'Position', [x y 80 30], 'Callback', @save_settings);
		end; % if win_num
	
		% Enables on all channels, all units
		x = fig1_width(1)-65; % -56
		yoffset = -92;
		y = fig1_height-65;
		w = 150;
		h = 20;
		data.ch1_checkbox(win_num) = uicontrol('style','checkbox','units','pixels',...
   		             'position',[x,y,w,h],'String','Enable',...
				'value',enable(win_num,1),'Callback',@ch1_enable);
		data.ch2_checkbox(win_num) = uicontrol('style','checkbox','units','pixels',...
		                'position',[x,y+yoffset,w,h],'String','Enable',...
				'value',enable(win_num,2),'Callback',@ch2_enable);
		data.ch3_checkbox(win_num) = uicontrol('style','checkbox','units','pixels',...
		                'position',[x,y+2*yoffset,w,h],'String','Enable',...
				'value',enable(win_num,3),'Callback',@ch3_enable);
		data.ch4_checkbox(win_num) = uicontrol('style','checkbox','units','pixels',...
		                'position',[x,y+3*yoffset,w,h],'String','Enable',...
				'value',enable(win_num,4),'Callback',@ch4_enable);
	end; % for win_num

% Make a variable exactly the size required
% Truncate or pad if necessary.
function out = exact_size(variable,rows,cols)
	out = variable;
	vrows = size(variable,1);
	vcols = size(variable,2);
	if vrows < rows
		% Add to rows
		out(vrows+1:rows,:) = zeros(rows-vrows,vcols);
	elseif vrows > rows
		out = out(1:rows,:);
	end;
	if vcols < cols
		% Add to cols
		out(:,vcols+1:cols) = zeros(rows,cols-vcols);
	elseif vcols > cols
		out = out(:,1:cols);
	end;
