%
% plot_saved_data(filename,unit_number,channel,feature)
% Version 1.1, 11/28/12 CMH
%
% Plots some aspects of data from a data file recorded in PicoMat_RTI
%   filename: name of MAT file with saved data
%   unit_number: which PicoScope unit you want to look at (1 to 4, usually 1)
%   channel: which PicoScope channel you want to look at (1 to 4)
%
% A quick hack by Prof. C M Higgins, 11/27/12
% Created for NROS 415
%
%
% Saved data includes:
%
% Parameters of data collection:
%  	BUFSIZE  		how many samples for one buffer of data from PicoScope
%  	FREQ_BIN_SIZE		Size in Hz of one frequency bin (smaller is better for EEG)
%  	sample_time		Time in seconds between samples.  Invert this for sample rate in Hz
%
% Metric parameters:
%	enable			2-D binary matrix, indexed by PicoScope unit number (rows), channel (cols)
%				    Tells whether each channel is enabled (1), or disabled (0)
%	channelsx		2-D matrix, indexed by channel (rows), PicoScope unit number (cols)
%				    Tells the mix of channels used to compute the horizontal metric
%	channelsy		2-D matrix, indexed by channel (rows), PicoScope unit number (cols)
%				    Tells the mix of channels used to compute the vertical metric
%	metricsx		1-D matrix, indexed by feature number (1-10).
%				    Tells the mix of features used to compute the horizontal metric
%	metricsy		1-D matrix, indexed by feature number (1-10).
%				    Tells the mix of features used to compute the vertical metric
%
% Features are: 
%    ampl env delta theta low_alpha high_alpha low_beta high_beta low_gamma high_gamma
%
% Parameters of control:
%	max1			Right maximum
%	max2			Left maximum
%	max3			Top maximum
%	max4			Bottom maximum
%	thresh1			Right threshold
%	thresh2			Left threshold
%	thresh3			Top threshold
%	thresh4			Bottom threshold
%
% Recorded data:
%	metricx			1-D matrix, horizontal metric versus time (samples)
%	metricy			1-D matrix, vertical metric versus time (samples)
%  	frequency_band_data 	4-D matrix, index 1: time,
%  				            index 2: unit_number,
%  				            index 3: channel,
%  				            index 4: frequency band (1-8).
function plot_saved_data(filename,unit_number,channel,feature)


if exist('filename') && exist(filename,'file');
	load(filename,'-mat');
else
	fprintf('Hmm, file doesn''t exist.\n');
	return;
end;

clc;
fprintf('Buffer size was %d samples.\n',BUFSIZE);
fprintf('Frequency bin size was %f Hz (integration time %f seconds).\n',FREQ_BIN_SIZE,1/FREQ_BIN_SIZE);
fprintf('Sample time was %f seconds (sample rate %s Hz).\n',sample_time,1/sample_time);
fprintf('%d PicoScope units were enabled.\n',size(enable,1));
fprintf('Enables:\n');
disp(enable);
fprintf('Channels X:\n');
disp(channelsx);
fprintf('Channels Y:\n');
disp(channelsy);
fprintf('Available features are:\n');
fprintf('1    2   3     4     5         6          7        8         9         10\n');
fprintf('ampl env delta theta low_alpha high_alpha low_beta high_beta low_gamma high_gamma\n');
fprintf('Metrics X:');
disp(metricsx);
fprintf('Metrics Y:');
disp(metricsy);

if ~exist('recorded_time')
	fprintf('I think this is just a settings file, so not displaying data.\n');
else

	figure(1);
	subplot(2,1,1);
	plot(recorded_time,recorded_metricx,'r',recorded_time,recorded_metricy,'g');
	ylabel('X metric (red), Y metric (green)');
	xlabel('Time (s)');
	subplot(2,1,2);
	plot(recorded_metricx,recorded_metricy);
	xlabel('X metric');
	ylabel('Y metric');
	axis([-max2 max1 -max4 max3]);
	x = -thresh2;
	y = -thresh4;
	w = thresh2+thresh1;
	h = thresh4+thresh3;
	rectangle('Position',[x y w h]);

	figure(2);
	x = size(recorded_freq_matrix,1);
	time = sample_time*(1:x);
	d = reshape(recorded_freq_matrix(:,unit_number,channel,feature),1,x);
size(time)
size(d)
	plot(time,d);
	xlabel('Time (s)');
	s = sprintf('Feature %d for unit %d channel %d',feature,unit_number,channel);
	ylabel(s);
end;
